# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
# $Id: install.py 1229 2011-01-29 17:07:16Z icemac $
"""Initial generation."""


def evolve(context):
    # nothing to install initially
    pass



# -*- coding: utf-8 -*-
# Copyright (c) 2011 Michael Howitz
# See also LICENSE.txt
import decimal
import icemac.addressbook.browser.testing
import icemac.addressbook.testing
import unittest2 as unittest


class TestEmptyNewValue(unittest.TestCase):
    """Testing handler for edge case that user enters empty new value."""

    layer = icemac.addressbook.browser.testing.WSGI_SEARCH_LAYER

    def test_adding_an_empty_new_value_does_not_change_the_updated_value(self):
        from icemac.addressbook.browser.search.result.handler.update.testing \
            import select_persons_with_keyword_for_update
        browser = select_persons_with_keyword_for_update('family')
        browser.getControl('field').displayValue = ['person -- last name']
        browser.getControl('Next').click()
        browser.getControl('new value', index=0).value = ''
        browser.getControl('operation').displayValue = [
            'append new value to existing one']
        browser.getControl('Next').click()
        # The last name column is displayed as a link column it contains the
        # unchanged last name:
        self.assertIn(
            '<td><a href="http://localhost/ab/Person-2">Koch</a></td>',
            browser.contents)


KEYWORD = u'keyword for test'


class TestUserDefinedFields(unittest.TestCase):
    """Testing update of user defined fields."""

    layer = icemac.addressbook.testing.WSGI_ADDRESS_BOOK_FUNCTIONAL_LAYER

    def setUp(self):
        self.ab = self.layer['rootFolder']['ab']

    def create_updateable_person(self, **kw):
        from icemac.addressbook.interfaces import IEntity, IPerson
        keywords = set(
            icemac.addressbook.testing.create_keyword(self.ab, keyword)
            for keyword in kw.pop('keywords', [KEYWORD]))
        data = {'last_name': u'Tester', 'keywords': keywords,
                'return_obj': True}
        data.update(kw)
        return icemac.addressbook.testing.create(
            self.ab, self.ab, IEntity(IPerson).class_name, **data)

    def test_bool_field_can_be_updated(self):
        from icemac.addressbook.interfaces import IEntity, IPerson
        from icemac.addressbook.browser.search.result.handler.update.testing \
            import select_persons_with_keyword_for_update
        field_name = icemac.addressbook.testing.create_field(
            self.ab, IEntity(IPerson).class_name, 'Bool', u'Ever met')
        self.create_updateable_person(**{field_name: False})
        browser = select_persons_with_keyword_for_update(KEYWORD)

        browser.getControl('field').displayValue = ['person -- Ever met']
        browser.getControl('Next').click()
        browser.getControl('yes').click()
        browser.getControl('operation').displayValue = [
            'replace existing value with new one']
        browser.getControl('Next').click()
        # Update sets the value to 'yes':
        self.assertIn('<td>Tester</td><td>yes</td>',
                      browser.contents.replace(' ', '').replace('\n', ''))

    def test_choice_field_can_be_updated(self):
        from icemac.addressbook.interfaces import IEntity, IPostalAddress
        address_class_name = IEntity(IPostalAddress).class_name
        field_name = icemac.addressbook.testing.create_field(
            self.ab, address_class_name, 'Choice', u'distance',
            values=[u'< 50 km', u'>= 50 km'])
        person = self.create_updateable_person()
        icemac.addressbook.testing.create(
            self.ab, person, address_class_name,
            **{field_name: '>= 50 km', 'set_as_default': True})
        from icemac.addressbook.browser.search.result.handler.update.testing \
            import select_persons_with_keyword_for_update
        browser = select_persons_with_keyword_for_update(KEYWORD)

        browser.getControl('field').displayValue = [
            'postal address -- distance']
        browser.getControl('Next').click()
        self.assertEqual(['no value', '< 50 km', '>= 50 km'],
                         browser.getControl('new value').displayOptions)
        browser.getControl('new value').displayValue = ['< 50 km']
        browser.getControl('operation').displayValue = [
            'replace existing value with new one']
        browser.getControl('Next').click()
        # Update sets the value to '< 50 km':
        self.assertIn('<td>Tester</td><td><50km</td>',
                      browser.contents.replace(' ', '').replace('\n', ''))

    def test_keywords_field_can_be_updated(self):
        from icemac.addressbook.interfaces import IEntity, IPerson
        person = self.create_updateable_person(
            keywords=[KEYWORD, u'second kw'])
        from icemac.addressbook.browser.search.result.handler.update.testing \
            import select_persons_with_keyword_for_update
        browser = select_persons_with_keyword_for_update(KEYWORD)

        browser.getControl('field').displayValue = ['person -- keywords']
        browser.getControl('Next').click()
        self.assertEqual([KEYWORD, 'second kw'],
                         browser.getControl('new value').displayOptions)
        browser.getControl('new value').displayValue = ['second kw']
        browser.getControl('operation').displayValue = [
            'remove selected keywords from existing ones']
        browser.getControl('Next').click()
        self.assertIn('<td>Tester</td><td>keywordfortest</td>',
                      browser.contents.replace(' ', '').replace('\n', ''))

    def _create_user_defined_field(self, field_type, field_class):
        """Create a user defined field."""
        from icemac.addressbook.interfaces import IEntity, IPostalAddress
        address_class_name = IEntity(IPostalAddress).class_name
        field_name = icemac.addressbook.testing.create_field(
            self.ab, address_class_name, field_type, u'distance')
        person = self.create_updateable_person()
        icemac.addressbook.testing.create(
            self.ab, person, address_class_name,
            **{field_name: field_class(50), 'set_as_default': True})

    def _update_field_value(self, field_name, operator, value):
        """Update a number field."""
        from icemac.addressbook.browser.search.result.handler.update.testing \
            import select_persons_with_keyword_for_update
        browser = select_persons_with_keyword_for_update(KEYWORD)

        browser.getControl('field').displayValue = [field_name]
        browser.getControl('Next').click()
        self.assertEqual('', browser.getControl('new value', index=0).value)
        browser.getControl('new value', index=0).value = value
        browser.getControl('operation').displayValue = [operator]
        browser.getControl('Next').click()
        return browser

    def _assert_number_field_can_be_updated(
            self, field_type, field_class, operator='add', value='5'):
        """Assert that a number field can be updated as expected."""
        self._create_user_defined_field(field_type, field_class)
        browser = self._update_field_value(
            'postal address -- distance', operator, value)
        # Update sets the value to 55:
        self.assertIn('<td>Tester</td><td>55</td>',
                      browser.contents.replace(' ', '').replace('\n', ''))

    def test_int_field_can_be_updated(self):
        self._assert_number_field_can_be_updated('Int', int)

    def test_decimal_field_can_be_updated(self):
        self._assert_number_field_can_be_updated('Decimal', decimal.Decimal)

    def test_validation_errors_show_up_in_result_table(self):
        from icemac.addressbook.interfaces import IEntity, IEMailAddress
        person = self.create_updateable_person()
        icemac.addressbook.testing.create(
            self.ab, person,
            IEntity(IEMailAddress).class_name, set_as_default=True)
        browser = self._update_field_value(
            'e-mail address -- e-mail address', 'append', 'foo')
        self.assertIn(
            '<td>Tester</td><td></td><td>fooisnotavalide-mailaddress.</td>',
            browser.contents.replace(' ', '').replace('\n', ''))
        # Complete button is not shown:
        self.assertEqual(
            ['form.buttons.back'],
            icemac.addressbook.testing.get_submit_control_names(browser))

    def test_division_by_zero_is_handled_like_a_validation_error(self):
        self._create_user_defined_field('Int', int)
        browser = self._update_field_value(
            'postal address -- distance', 'div', '0')
        self.assertIn('<td>Tester</td><td>50</td><td>Divisionbyzero</td>',
                      browser.contents.replace(' ', '').replace('\n', ''))
        # Complete button is not shown:
        self.assertEqual(
            ['form.buttons.back'],
            icemac.addressbook.testing.get_submit_control_names(browser))

    def test_datatype_of_field_for_change_can_be_changed(self):
        self.create_updateable_person()
        browser = self._update_field_value(
            'person -- first name', 'append', 'foo')
        browser.getControl('Back').click()
        browser.getControl('Back').click()
        browser.getControl('field').displayValue = ['person -- birth date']
        browser.getControl('Next').click()
        # Another field is used to avoid conflicts on data types:
        self.assertEqual('', browser.getControl('new value').value)

    def test_not_hitting_complete_button_does_not_persist_any_changes(self):
        self.create_updateable_person()
        browser = self._update_field_value(
            'person -- last name', 'append', 'foo')
        browser.getLink('Person list').click()
        # The last name of person 'Tester' is unchanged:
        self.assertIn('<a href="http://localhost/ab/Person">Tester</a>',
                      browser.contents)

    def test_if_user_selects_a_step_with_data_missing_he_gets_redirected_back(
            self):
        self.create_updateable_person()
        from icemac.addressbook.browser.search.result.handler.update.testing \
            import select_persons_with_keyword_for_update
        browser = select_persons_with_keyword_for_update(KEYWORD)
        self.assertEqual('http://localhost/ab/@@multi-update', browser.url)
        browser.getLink('New value').click()
        # 'chooseField' is the first step, so we get redirected there
        self.assertEqual(
            'http://localhost/ab/multi-update/chooseField', browser.url)
        browser.getLink('Check result').click()
        self.assertEqual(
            'http://localhost/ab/multi-update/chooseField', browser.url)
        browser.getControl('field').displayValue = ['person -- first name']
        browser.getControl('Next').click()

        self.assertEqual(
            'http://localhost/ab/multi-update/enterValue', browser.url)
        browser.getLink('Choose field').click()
        self.assertEqual(
            'http://localhost/ab/multi-update/chooseField', browser.url)
        browser.getLink('Check result').click()
        # After selecting the field 'enterValue' is complete, as the only
        # required field has a default value
        self.assertEqual(
            'http://localhost/ab/multi-update/checkResult', browser.url)
        # There is no 'complete' button as the user did not enter data in step
        # 'enterValue':
        self.assertEqual(
            ['form.buttons.back'],
            icemac.addressbook.testing.get_submit_control_names(browser))



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2011 Michael Howitz
# See also LICENSE.txt

import hurry.jqueryui
import hurry.resource
import os.path

# CSS
css = hurry.resource.Library('css', os.path.join('resources', 'css'))


reset_css = hurry.resource.ResourceInclusion(css, 'reset.css')
base_css = hurry.resource.ResourceInclusion(
    css, 'base.css', depends=[reset_css])
table_css = hurry.resource.ResourceInclusion(css, 'table.css')
form_css = hurry.resource.ResourceInclusion(css, 'form.css')
no_max_content_css = hurry.resource.ResourceInclusion(
    css, 'no_max_content.css', depends=[form_css])
import_css = hurry.resource.ResourceInclusion(css, 'wizard.css')

# JavaScript
js = hurry.resource.Library('js', os.path.join('resources', 'js'))

masterdata_fields = hurry.resource.ResourceInclusion(
    js, 'masterdata_fields.js', depends=[hurry.jqueryui.jqueryui])



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2011 Michael Howitz
# See also LICENSE.txt
import datetime
import icemac.addressbook.browser.table
import icemac.addressbook.entities
import icemac.addressbook.fieldsource
import icemac.addressbook.interfaces
import icemac.truncatetext
import z3c.form.term
import z3c.table.column
import zope.component
import zope.i18n
import zope.preference.interfaces
import zope.schema.interfaces


END_OF_DATE = datetime.date(datetime.MAXYEAR, 12, 31)
END_OF_DATETIME = datetime.datetime(datetime.MAXYEAR, 12, 31, 23, 59, 59)


class BaseColumn(z3c.table.column.Column):
    """Column which is able to get an object on an attribute of the item and
    adapt it to specified interface."""

    entity = None  # referenced entity
    field = None  # referenced field on entity
    defaultValue = u''  # Value for display when there is no value.

    def getRawValue(self, item):
        "Compute the value, which can be None."
        schema_field = icemac.addressbook.entities.get_bound_schema_field(
            item, self.entity, self.field)
        # Returning the value of the bound object as it might differ from item:
        return schema_field.get(schema_field.context)

    def getValue(self, item):
        "Compute the value, mostly ready for display."
        obj = self.getRawValue(item)
        if obj is None:
            return self.defaultValue
        return obj

    renderCell = getValue


class LinkColumn(z3c.table.column.LinkColumn,
                 BaseColumn):
    """LinkColumn which fetches link content from item."""

    def getSortKey(self, item):
        # Sort by the value displayed to the user, but lowercase it,
        # as the user does not expect lower case names below the
        # 'normal' upper case ones.
        return self.getLinkContent(item).lower()

    def getLinkContent(self, item):
        # Get the content from item or return the default value when empty.
        return self.getValue(item)

    def renderCell(self, item):
        if self.getLinkContent(item):
            return super(LinkColumn, self).renderCell(item)
        return self.defaultValue


class URLColumn(LinkColumn):
    """LinkColumn where the link URL is the same as the link content."""

    getLinkURL = LinkColumn.getLinkContent


class DateTimeColumn(z3c.table.column.FormatterColumn,
                 BaseColumn):
    """Column which is sortable even with `None` values in it."""

    maxValue = END_OF_DATETIME

    def renderCell(self, item):
        value = self.getRawValue(item)
        if value:
            return self.getFormatter().format(value)
        return self.defaultValue

    def getSortKey(self, item):
        key = self.getRawValue(item)
        if key is None:
            # empty date fields should be sorted to the end of the list
            key = self.maxValue
        return key


class DateColumn(DateTimeColumn):
    "DateColumn which is able to sort even `None` values."

    formatterCategory = 'date'
    maxValue = END_OF_DATE


class TruncatedContentColumn(BaseColumn):
    """Column which truncates its content."""

    length = 20  # number of characters to display
    ellipsis = u'…'  # ellipsis sign

    def renderCell(self, item):
        return icemac.truncatetext.truncate(
            self.getRawValue(item), self.length, self.ellipsis)


class BoolColumn(BaseColumn):
    "AdaptedGetAttrColumn for displaying bool values."

    def renderCell(self, item):
        value = self.getRawValue(item)
        # We use the labels of z3c.form here so the displayed values
        # are the same as in the edit form.
        if value is True:
            label = z3c.form.term.BoolTerms.trueLabel
        elif value is False:
            label = z3c.form.term.BoolTerms.falseLabel
        else:
            return self.defaultValue
        return zope.i18n.translate(label, context=self.request)


class TranslatedTiteledColumn(BaseColumn):
    """Column which returns the translated ITitle of the value."""

    def renderCell(self, obj):
        "Get the title of the value."
        value = self.getRawValue(obj)
        translated = self.defaultValue
        if value:
            title = icemac.addressbook.interfaces.ITitle(value)
            translated = zope.i18n.translate(title, context=self.request)
        return translated


class EMailColumn(BaseColumn,
                  z3c.table.column.EMailColumn):
    "Column which renders the cell contents as mailto-link."

    renderCell = z3c.table.column.EMailColumn.renderCell


def getColumnClass(entity, field):
    """Get a column class to display the requested field of an entity."""
    if field.__name__ in ('first_name', 'last_name'):
        # Person's first name and last name should be links:
        return LinkColumn
    if field.__name__ == 'keywords':
        # Keywords need a special column as they are an iterable:
        return icemac.addressbook.browser.table.KeywordsColumn
    if (zope.schema.interfaces.IText.providedBy(field) and
        not zope.schema.interfaces.ITextLine.providedBy(field)):
        # The content of text areas (but not text lines, which extend from
        # text area) should get truncated.
        return TruncatedContentColumn
    if zope.schema.interfaces.IDate.providedBy(field):
        # Date fields need a special column, as None values are not
        # compareable to date values:
        return DateColumn
    if field.__name__ == 'country':
        # Country is an object, so the translated title should be displayed:
        return TranslatedTiteledColumn
    if field.__name__ == 'email':
        # The e-mail address should be a mailto-link:
        return EMailColumn
    if field.__name__ == 'url':
        return URLColumn
    if icemac.addressbook.interfaces.IField.providedBy(field):
        # User defined fields:
        if field.type == u'Bool':
            return BoolColumn
        if field.type == u'URI':
            return URLColumn
        if field.type == u'Text':
            return TruncatedContentColumn
        if field.type == u'Date':
            return DateColumn
        if field.type == u'Datetime':
            return DateTimeColumn
    return BaseColumn


def createFieldColumn(table, entity, field, weight):
    """Create a column for the specified `entity` and `field`."""
    additional_column_args = dict()
    if (field.__name__ == 'url' or
        (icemac.addressbook.interfaces.IField.providedBy(field) and
         field.type == 'URI')):
        additional_column_args['linkTarget'] = '_blank'

    return z3c.table.column.addColumn(
        table, getColumnClass(entity, field), field.__name__,
        header=field.title, weight=weight,
        entity=entity, field=field, **additional_column_args)


class BasePersonList(object):
    """List persons in address book without batching.

    Either extend `icemac.addressbook.browser.table.PageletTable`
    or `icemac.addressbook.browser.table.Table`.

    """
    @property
    def prefs(self):
        "User defined preferences for person list."
        return zope.component.getUtility(
            zope.preference.interfaces.IPreferenceGroup, name="personList")

    def __init__(self, *args, **kw):
        super(BasePersonList, self).__init__(*args, **kw)
        self.sortOrder = self.prefs.sort_direction
        self._columns = self._set_up_columns()

    def _set_up_columns(self):
        """Creates the columns of the table."""
        zcml_columns = super(BasePersonList, self).setUpColumns()
        zcml_columns_before = len(
            [x for x in zcml_columns if z3c.table.table.getWeight(x) < 0])
        python_columns = self._set_up_user_selected_columns_and_sort_on(
            zcml_columns_before)
        return zcml_columns + python_columns

    def _set_up_user_selected_columns_and_sort_on(self, columns_before):
        """Creates the columns selected by the user in the preferences and
        computes the sort order."""
        prefs = self.prefs
        order_by = prefs.order_by
        columns = []
        index = 0  # current column index
        # Entity and field of the column which sould be used for order-by:
        try:
            order_by_entity, order_by_field = (
                icemac.addressbook.fieldsource.untokenize(order_by))
        except KeyError:
            # Field has been deleted, so we can't use it for sorting:
            order_by_entity, order_by_field = None, None

        # Create all columns and compute the order-by column:
        for column_name in self.prefs.columns:
            try:
                entity, field = (
                    icemac.addressbook.fieldsource.untokenize(column_name))
            except KeyError:
                # Column no longer exists
                continue
            columns.append(createFieldColumn(self, entity, field, index))
            if entity == order_by_entity and field == order_by_field:
                # Found the entity and field for order by, so set the
                # expected name on the sortOn variable. The last parameter
                # depends on the position of the column in the table when
                # there are columns before the columns defined by this
                # method the index gets higher:
                self.sortOn = '%s-%s-%s' % (
                    self.prefix, field.__name__, index + columns_before)
            # current column exists, so the index can be adjusted
            index += 1
        return columns

    def setUpColumns(self):
        "Return the previously computed columns and via ZCML registered ones."
        return self._columns

    values = NotImplemented



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
# $Id: __init__.py 1356 2011-10-28 20:00:27Z icemac $


class Search(object):
    """View to select a search."""
    show_headline = True
    form_explanation = u''



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
# $Id: menu.py 1229 2011-01-29 17:07:16Z icemac $

import zope.viewlet.manager
import icemac.addressbook.browser.menus.interfaces
import z3c.menu.ready2go.manager

MainMenu = zope.viewlet.manager.ViewletManager(
    'main-menu', icemac.addressbook.browser.menus.interfaces.IMainMenu,
    bases=(z3c.menu.ready2go.manager.MenuManager,))


def getWeight((name, viewlet)):
    view_name = viewlet.viewName
    if view_name.startswith('@@'):
        # remove starting @@
        view_name = view_name[2:]
    view = zope.component.getMultiAdapter(
        (viewlet.context, viewlet.request), name=view_name)
    interface = getattr(view, 'interface', None)
    if interface is None:
        interface = getattr(view, 'interface_for_menu', None)
    assert interface is not None, (
        'View %r does neither have a non-null value on `interface` nor '
        'on `interface_for_menu` attribute. But this is needed to compute '
        'the AddMenu contents.' % view)
    entity = icemac.addressbook.interfaces.IEntity(interface)
    order = zope.component.getUtility(
        icemac.addressbook.interfaces.IEntityOrder)
    try:
        return order.get(entity)
    except (KeyError, zope.component.ComponentLookupError):
        # The entity is not known in the order or we are outside an address
        # book.
        return 0


class OrdersWeightMenuManager(z3c.menu.ready2go.manager.MenuManager):
    "Menu manager which uses address_book.orders as weight."

    def sort(self, viewlets):
        return sorted(viewlets, key=getWeight)


AddMenu = zope.viewlet.manager.ViewletManager(
    'add-menu', icemac.addressbook.browser.menus.interfaces.IAddMenu,
    bases=(OrdersWeightMenuManager,))



import icemac.addressbook.browser.base
import zope.component.hooks


class MailTo(object):

    def get_persons(self):
        addressbook = zope.component.hooks.getSite()
        session = icemac.addressbook.browser.base.get_session(self.request)
        for id in session['person_ids']:
            yield addressbook[id]

    @property
    def unique_mail_addresses(self):
        addresses = set()
        for person in self.get_persons():
            email = icemac.addressbook.interfaces.IEMailAddress(person, None)
            if email is None:
                continue
            if not email.email:
                continue
            addresses.add(email.email)
        return sorted(addresses)

    def mailto_link(self):
        return 'mailto:?bcc=%s' % ','.join(self.unique_mail_addresses)



# -*- coding: utf-8 -*-
# Copyright (c) 2011 Michael Howitz
# See also LICENSE.txt


def FieldSerializer(field, obj):
    """Serializes the value of a field to a unicode value."""
    return unicode(field.get(obj))


def FieldNoSerializer(field, obj):
    """Serializes the value of a field to the empty string."""
    return u''



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2011 Michael Howitz
# See also LICENSE.txt

import BTrees.OOBTree
import icemac.addressbook.interfaces
import persistent
import persistent.list
import zope.container.contained
import zope.interface


class OrderStorage(
    persistent.Persistent, zope.container.contained.Contained):
    """Storage of orders of objects."""

    zope.interface.implements(icemac.addressbook.interfaces.IOrderStorage)

    def __init__(self, *args, **kw):
        self._storage = BTrees.OOBTree.OOBTree()

    # IOrderStorageRead
    def namespaces(self):
        """Get an iterable of the known namespaces."""
        return self._storage.keys()

    def get(self, obj, namespace):
        """Get the index of the object in the list."""
        by_namespace = self.__iter__(namespace)
        try:
            return by_namespace.index(obj)
        except ValueError:
            raise KeyError(obj)

    def isFirst(self, obj, namespace):
        """Tell whether `obj` is the first object in the list."""
        return self.get(obj, namespace) == 0

    def isLast(self, obj, namespace):
        """Tell whether `obj` is the last object in the list."""
        return (self.get(obj, namespace) + 1) == len(self.__iter__(namespace))

    def __iter__(self, namespace):
        """Iterate over the list of a namespace."""
        return self._storage[namespace]

    # IOrderStorageWrite

    def add(self, obj, namespace):
        """Add an object to the order for a namespace."""
        if namespace not in self._storage:
            self._create_namespace(namespace)
        storage = self._storage[namespace]
        if obj not in storage:
            storage.append(obj)

    def remove(self, obj, namespace):
        """Remove the object from the order of a namespace."""
        self._storage[namespace].remove(obj)

    def truncate(self, namespace):
        """Remove all objects from the order of a namespace."""
        if namespace in self._storage:
            # Creates new empty namespace when it exists.
            self._create_namespace(namespace)

    def up(self, obj, namespace, delta=1):
        """Move the object one position up in the list."""
        storage = self._storage[namespace]
        for i in xrange(delta):
            index = storage.index(obj)
            if index == 0:
                raise ValueError(
                    "Moving %r by %s positions up would move it beyond the "
                    "beginning of the list." % (obj, delta))
            storage[index - 1:index + 1] = reversed(
                storage[index - 1:index + 1])

    def down(self, obj, namespace, delta=1):
        """Move the object one position down in the list."""
        storage = self._storage[namespace]
        for i in xrange(delta):
            index = storage.index(obj)
            if index == len(storage) - 1:
                raise ValueError(
                    "Moving %r by %s positions down would move it beyond the "
                    "end of the list." % (obj, delta))
            storage[index:index + 2] = reversed(storage[index:index + 2])

    # private

    def _create_namespace(self, namespace):
        self._storage[namespace] = persistent.list.PersistentList()



# -*- coding: utf-8 -*-
# Copyright (c) 2011 Michael Howitz
# See also LICENSE.txt
from __future__ import absolute_import
import mock
import unittest2 as unittest


class Test_update_persons(unittest.TestCase):
    """Testing .base.update_persons."""

    def setUp(self):
        import grokcore.component.testing
        import zope.annotation.attribute
        import zope.component
        import zope.component.testing
        base = 'icemac.addressbook.browser.search.result.handler.update'
        grokcore.component.testing.grok(base + '.operators')
        grokcore.component.testing.grok(base + '.errormessages')
        zope.component.provideAdapter(
            zope.annotation.attribute.AttributeAnnotations)
        self.addCleanup(zope.component.testing.tearDown)

    def test_updates_only_specified_field(self):
        from icemac.addressbook.person import Person, person_entity
        from ..base import update_persons
        person1 = Person()
        person1.notes = u'p1'
        person2 = Person()
        person2.notes = u'p2'
        update_persons((person1, person2), person_entity,
                       person_entity.getRawField('notes'), 'append', u'bar')
        self.assertEqual('p1bar', person1.notes)
        self.assertEqual('p2bar', person2.notes)
        # no other attributes are in vars as they are not changed (only the
        # private container attributes are additionally here).
        self.assertEqual(
            ['notes', '_BTreeContainer__len', '_SampleContainer__data'],
            vars(person1).keys())

    @mock.patch('icemac.addressbook.browser.search.result.handler.update.'
                'operators.NoneAppend.__call__')
    def test_returns_custom_ZeroDivisionError_if_division_by_zero_occurred(
            self, op):
        from icemac.addressbook.person import Person, person_entity
        from ..base import update_persons
        op.side_effect = ZeroDivisionError
        person1 = Person()
        person1.__name__ = 'person1'
        result = update_persons(
            [person1], person_entity,
            person_entity.getRawField('notes'), 'append', u'bar')
        self.assertEqual({'person1': u'Division by zero'}, result)

    @mock.patch('icemac.addressbook.browser.search.result.handler.update.'
                'operators.NoneAppend.__call__')
    def test_returns_custom_Exception_if_other_exception_occurred(self, op):
        from ..base import update_persons
        from icemac.addressbook.person import Person, person_entity
        import zope.i18n
        op.side_effect = IOError('file not found')
        person1 = Person()
        person1.__name__ = 'person1'
        result = update_persons(
            [person1], person_entity,
            person_entity.getRawField('notes'), 'append', u'bar')
        self.assertEqual(
            [('person1',
              u'Unexpected error occurred: IOError: file not found')],
            [(key, zope.i18n.translate(val)) for key, val in result.items()])



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt

import icemac.addressbook.addressbook
import icemac.addressbook.interfaces
import icemac.addressbook.testing
import icemac.addressbook.utils
import unittest
import zope.authentication.interfaces
import zope.catalog.interfaces
import zope.intid.interfaces
import zope.location.interfaces
import zope.pluggableauth.interfaces


class TestAddressbook(unittest.TestCase):

    layer = icemac.addressbook.testing.FUNCTIONAL_LAYER

    def assertLocalUtility(self, ab, iface):
        self.assertTrue(icemac.addressbook.utils.utility_locally_registered(
            ab, iface))

    def assertAttribute(self, ab, attribute, iface):
        self.assertTrue(iface.providedBy(getattr(ab, attribute)))
        self.assertLocalUtility(ab, iface)

    def check_addressbook(self, ab):
        self.assertTrue(zope.location.interfaces.ISite.providedBy(ab))
        self.assertAttribute(
            ab, 'keywords', icemac.addressbook.interfaces.IKeywords)
        self.assertAttribute(
            ab, 'principals',
            zope.pluggableauth.interfaces.IAuthenticatorPlugin)
        self.assertAttribute(
            ab, 'entities',
            icemac.addressbook.interfaces.IEntities)
        self.assertAttribute(
            ab, 'orders',
            icemac.addressbook.interfaces.IOrderStorage)
        self.assertLocalUtility(ab, zope.intid.interfaces.IIntIds)
        self.assertLocalUtility(ab, zope.catalog.interfaces.ICatalog)
        self.assertLocalUtility(
            ab, zope.authentication.interfaces.IAuthentication)

    def setUp(self):
        super(TestAddressbook, self).setUp()
        root = self.layer['rootFolder']
        root['ab'] = self.ab = icemac.addressbook.utils.create_obj(
            icemac.addressbook.addressbook.AddressBook)

    def test_create(self):
        self.check_addressbook(self.ab)

    def test_recall_create_infrastructure(self):
        icemac.addressbook.addressbook.create_address_book_infrastructure(
            self.ab)
        self.check_addressbook(self.ab)

    def test___repr___no___name__(self):
        self.assertEqual("<AddressBook None (None)>",
                         repr(icemac.addressbook.addressbook.AddressBook()))

    def test___repr___no_title(self):
        self.assertEqual("<AddressBook u'ab' (None)>", repr(self.ab))

    def test___repr__(self):
        self.ab.title = u'My address book'
        self.assertEqual("<AddressBook u'ab' (u'My address book')>",
                         repr(self.ab))



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
# $Id: evolve10.py 1229 2011-01-29 17:07:16Z icemac $

import icemac.addressbook.generations.utils


def evolve(context):
    """Install default preferences provider.
    """
    icemac.addressbook.generations.utils.update_address_book_infrastructure(
        context)



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
# $Id: addressbook.py 1427 2011-12-06 07:15:23Z icemac $

from icemac.addressbook.i18n import _
import grokcore.component
import icemac.addressbook.entities
import icemac.addressbook.file.interfaces
import icemac.addressbook.interfaces
import icemac.addressbook.keyword
import icemac.addressbook.orderstorage
import icemac.addressbook.preferences.default
import icemac.addressbook.utils
import zc.catalog.catalogindex
import zope.app.appsetup.bootstrap
import zope.authentication.interfaces
import zope.catalog.catalog
import zope.catalog.interfaces
import zope.catalog.text
import zope.component
import zope.container.btree
import zope.container.interfaces
import zope.event
import zope.index.text.lexicon
import zope.interface
import zope.intid
import zope.intid.interfaces
import zope.location
import zope.location.interfaces
import zope.pluggableauth.authentication
import zope.pluggableauth.interfaces
import zope.pluggableauth.plugins.principalfolder
import zope.schema.fieldproperty
import zope.site.site


class AddressBook(zope.container.btree.BTreeContainer,
                  zope.site.site.SiteManagerContainer):
    "An address book."

    zope.interface.implements(icemac.addressbook.interfaces.IAddressBook)

    entities = None
    importer = None
    keywords = None
    principals = None
    orders = None

    title = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IAddressBook['title'])

    def __repr__(self):
        return "<AddressBook %r (%r)>" % (self.__name__, self.title)

address_book_entity = icemac.addressbook.entities.create_entity(
    _(u'address book'),
    icemac.addressbook.interfaces.IAddressBook, AddressBook)


def create_and_register(addressbook, attrib_name, class_, interface, name=''):
    """Create an object on an attribute and register it as local utility."""
    if getattr(addressbook, attrib_name) is not None:
        return
    site_mgr = addressbook.getSiteManager()
    obj = icemac.addressbook.utils.create_obj(class_)
    setattr(addressbook, attrib_name, obj)
    zope.location.locate(obj, addressbook, '++attribute++' + attrib_name)
    site_mgr.registerUtility(obj, interface, name=name)


@zope.component.adapter(
    icemac.addressbook.interfaces.IAddressBook,
    zope.container.interfaces.IObjectAddedEvent)
def create_address_book_infrastructure(addressbook, event=None):
    """Create initial infrastructure or update existing infrastructure to
    current requirements (using generation)."""
    # add site manager
    if not zope.location.interfaces.ISite.providedBy(addressbook):
        site_manager = zope.site.site.LocalSiteManager(addressbook)
        addressbook.setSiteManager(site_manager)

    # add keywords container
    create_and_register(
        addressbook, 'keywords', icemac.addressbook.keyword.KeywordContainer,
        icemac.addressbook.interfaces.IKeywords)

    # add principals folder
    create_and_register(
        addressbook, 'principals',
        zope.pluggableauth.plugins.principalfolder.PrincipalFolder,
        zope.pluggableauth.interfaces.IAuthenticatorPlugin,
        name=u'icemac.addressbook.principals')

    # add entities utility
    create_and_register(
        addressbook, 'entities',
        icemac.addressbook.entities.PersistentEntities,
        icemac.addressbook.interfaces.IEntities)

    # add order storage utility
    create_and_register(
        addressbook, 'orders',
        icemac.addressbook.orderstorage.OrderStorage,
        icemac.addressbook.interfaces.IOrderStorage)

    def add_entity_to_order(iface):
        addressbook.orders.add(
            icemac.addressbook.interfaces.IEntity(iface).name,
            icemac.addressbook.interfaces.ENTITIES)

    add_entity_to_order(icemac.addressbook.interfaces.IAddressBook)
    add_entity_to_order(icemac.addressbook.interfaces.IPerson)
    add_entity_to_order(icemac.addressbook.interfaces.IPersonDefaults)
    add_entity_to_order(icemac.addressbook.interfaces.IPostalAddress)
    add_entity_to_order(icemac.addressbook.interfaces.IPhoneNumber)
    add_entity_to_order(icemac.addressbook.interfaces.IEMailAddress)
    add_entity_to_order(icemac.addressbook.interfaces.IHomePageAddress)
    add_entity_to_order(icemac.addressbook.file.interfaces.IFile)
    add_entity_to_order(icemac.addressbook.interfaces.IKeyword)

    zope.event.notify(AddressBookCreated(addressbook))


class AddressBookCreated(object):
    """Event which signales that an address book has been created.

    Subscribers can expect that the new address book is a site (but not set
    as such).
    """

    def __init__(self, address_book):
        self.address_book = address_book


@grokcore.component.subscribe(AddressBookCreated)
def add_more_addressbook_infrastructure(event):
    "Add more infrastructure which depends on addressbook set as site."
    addressbook = event.address_book
    try:
        old_site = zope.component.hooks.getSite()
        zope.component.hooks.setSite(addressbook)
        # intid utility
        if not icemac.addressbook.utils.utility_locally_registered(
                addressbook, zope.intid.interfaces.IIntIds):
            intids = zope.app.appsetup.bootstrap.ensureUtility(
                addressbook, zope.intid.interfaces.IIntIds, '',
                zope.intid.IntIds)

        # catalog
        if icemac.addressbook.utils.utility_locally_registered(
                addressbook, zope.catalog.interfaces.ICatalog):
            catalog = zope.component.queryUtility(
                zope.catalog.interfaces.ICatalog)
        else:
            # add catalog
            catalog = zope.app.appsetup.bootstrap.ensureUtility(
                addressbook, zope.catalog.interfaces.ICatalog, '',
                zope.catalog.catalog.Catalog)

        # indexes
        if 'keywords' not in catalog:
            catalog['keywords'] = zc.catalog.catalogindex.SetIndex(
                'get_titles', icemac.addressbook.interfaces.IKeywordTitles,
                field_callable=True)
        if 'name' not in catalog:
            catalog['name'] = zope.catalog.text.TextIndex(
                'get_name', icemac.addressbook.interfaces.IPersonName,
                field_callable=True,
                lexicon=zope.index.text.lexicon.Lexicon(
                    zope.index.text.lexicon.Splitter(),
                    zope.index.text.lexicon.CaseNormalizer()))

        # authenticator (PAU)
        if not icemac.addressbook.utils.utility_locally_registered(
                addressbook,
                zope.authentication.interfaces.IAuthentication):
            pau = zope.app.appsetup.bootstrap.ensureUtility(
                addressbook, zope.authentication.interfaces.IAuthentication,
                '',
                zope.pluggableauth.authentication.PluggableAuthentication)
            pau.credentialsPlugins = (u'No Challenge if Authenticated',
                                      u'Flashed Session Credentials',)
            pau.authenticatorPlugins = (u'icemac.addressbook.principals',)

        # default preferences
        icemac.addressbook.preferences.default.add(addressbook)
    finally:
        zope.component.hooks.setSite(old_site)



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2011 Michael Howitz
# See also LICENSE.txt

import doctest
import icemac.addressbook.namechooser.namechooser
import plone.testing
import zope.annotation.attribute
import zope.testing.cleanup
import zope.testrunner.layer


class NameChooserLayer(plone.testing.Layer):
    """Layer for name chooser tests."""
    defaultBases = (zope.testrunner.layer.UnitTests,)

    def setUp(self):
        zope.component.provideAdapter(
            zope.annotation.attribute.AttributeAnnotations)
        zope.component.provideAdapter(
            icemac.addressbook.namechooser.namechooser.name_suffix)

    def tearDown(self):
        zope.testing.cleanup.tearDown()

NAME_CHOOSER_LAYER = NameChooserLayer()


def test_suite():
    suite = doctest.DocFileSuite('namechooser.txt')
    suite.layer = NAME_CHOOSER_LAYER
    return suite



import icemac.addressbook.browser.testing
import icemac.addressbook.testing


def test_suite():
    return icemac.addressbook.testing.DocFileSuite(
        "update.txt",
        package="icemac.addressbook.browser.search.result.handler.update",
        layer=icemac.addressbook.browser.testing.WSGI_SEARCH_LAYER)



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
# $Id: table.py 1296 2011-08-10 18:50:51Z icemac $

from icemac.addressbook.i18n import MessageFactory as _
import icemac.addressbook.browser.resource
import icemac.addressbook.interfaces
import icemac.truncatetext
import z3c.pagelet.browser
import z3c.table.batch
import z3c.table.column
import z3c.table.table


# Columns

class TitleLinkColumn(z3c.table.column.LinkColumn):
    """Column containing the title of an object and a link to the object."""

    header = _(u'Name')
    weight = 2

    def getSortKey(self, item):
        return icemac.addressbook.interfaces.ITitle(item).lower()

    def getLinkContent(self, item):
        return icemac.addressbook.interfaces.ITitle(item)


class DeleteLinkColumn(z3c.table.column.LinkColumn):
    """Column containing the a link to delete the object."""

    header = _(u'')
    weight = 100
    linkContent = _(u'Delete')
    linkName = '@@delete.html'


class TruncatedContentColumn(z3c.table.column.GetAttrColumn):
    "Column which truncates its content to `length` characters."

    length = 20  # number of characters to display
    attrName = None  # attribute to access
    ellipsis = u'…'  # ellipsis sign
    defaultValue = u''  # default value when there is no value

    def getValue(self, obj):
        value = super(TruncatedContentColumn, self).getValue(obj)
        if value is None:
            return self.defaultValue
        result = icemac.truncatetext.truncate(
            value, self.length, self.ellipsis)
        return result


class KeywordsColumn(z3c.table.column.GetAttrColumn):
    """GetAttrColumn where attr is an iterable of keywords."""

    attrName = 'keywords'

    def getSortKey(self, item):
        return super(KeywordsColumn, self).getSortKey(item).lower()

    def getValue(self, obj):
        values = super(KeywordsColumn, self).getValue(obj)
        return u', '.join(sorted(
            (icemac.addressbook.interfaces.ITitle(x) for x in values),
            key=lambda x: x.lower()))


class LinkColumn(z3c.table.column.LinkColumn):
    "Special LinkColumn which does not display a link when the URL is `None`."

    defaultValue = u''  # value which is rendered when there is no URL

    def renderCell(self, item):
        if not self.getLinkURL(item):
            return self.defaultValue
        return super(LinkColumn, self).renderCell(item)


# Tables

class Table(z3c.table.table.Table):
    "Table which supports a no-rows-found message."

    cssClassEven = u'table-even-row'
    cssClassOdd = u'table-odd-row'
    startBatchingAt = 1000000
    no_rows_message = u''  # Set at subclass.

    def update(self):
        icemac.addressbook.browser.resource.table_css.need()
        super(Table, self).update()

    def renderTable(self):
        if self.rows:
            return super(Table, self).renderTable()
        return self.no_rows_message


class PageletTable(z3c.pagelet.browser.BrowserPagelet, Table):
    """Render the table in a pagelet which also has a template.

    When no template is required the `Table` class can be used.
    """

    def __init__(self, *args, **kw):
        super(PageletTable, self).__init__(*args, **kw)
        Table.__init__(self, *args, **kw)
    update = Table.update


# Batching

class NiceBatchProvider(z3c.table.batch.BatchProvider):
    """A batch provider with a nicer spacer."""

    batchSpacer = u'…'



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
# $Id: test_doctests.py 1255 2011-03-05 08:25:37Z icemac $

import icemac.addressbook.testing


def test_suite():
    return icemac.addressbook.testing.FunctionalDocFileSuite(
        # Caution: none of these tests can run as unittest!
        'catalog.txt',
        package="icemac.addressbook.catalog",
        )



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2011 Michael Howitz
# See also LICENSE.txt

from icemac.addressbook.i18n import MessageFactory as _
import icemac.addressbook.browser.metadata
import icemac.addressbook.entities
import icemac.addressbook.interfaces
import urlparse
import zope.app.publication.traversers
import zope.publisher.interfaces
import zope.publisher.interfaces.http
import zope.security.proxy
import zope.traversing.browser
import icemac.addressbook.browser.resource


class FieldsTraverser(
    zope.app.publication.traversers.SimpleComponentTraverser):

    zope.interface.implementsOnly(zope.publisher.interfaces.IPublishTraverse)
    zope.component.adapts(icemac.addressbook.interfaces.IEntity,
                          zope.publisher.interfaces.http.IHTTPRequest)

    def publishTraverse(self, request, name):
        entities = zope.component.queryUtility(
            icemac.addressbook.interfaces.IEntities)
        try:
            return entities[name]
        except KeyError:
            return super(FieldsTraverser, self).publishTraverse(request, name)


def get_field_URL(entity, field, request):
    """Compute the URL to access a field."""
    entities = zope.component.getUtility(
        icemac.addressbook.interfaces.IEntities)
    url = zope.traversing.browser.absoluteURL(entities, request)
    url += "/" + entity.__name__ + "/" + field.__name__
    return url


class List(object):
    """List fields of an entity."""

    def update(self):
        super(List, self).update()
        icemac.addressbook.browser.resource.masterdata_fields.need()
        icemac.addressbook.browser.resource.table_css.need()
        icemac.addressbook.browser.resource.form_css.need()

    def _values(self):
        # zope.schema fields are no content classes, so they have no
        # permissions defined
        return [zope.security.proxy.getObject(field)
                for name, field in self.context.getRawFields()]

    def fields(self):
        for field in self._values():
            if icemac.addressbook.interfaces.IField.providedBy(field):
                url = get_field_URL(self.context, field, self.request)
                delete_url = '%s/@@delete.html' % url
            else:
                url = delete_url = None
            yield {'title': field.title,
                   'delete-link': delete_url,
                   'edit-link': url,
                   'id': field.__name__}


class SaveSortorder(icemac.addressbook.browser.base.BaseView):
    """Save the field sort order as defined by user."""

    def __call__(self, f):
        self.context.setFieldOrder(f)
        self.send_flash(_('Saved sortorder.'))
        self.request.response.redirect(self.url(self.context))


class AddForm(icemac.addressbook.browser.base.BaseAddForm):

    class_ = icemac.addressbook.entities.Field
    interface = icemac.addressbook.interfaces.IField
    next_url = 'parent'

    def add(self, obj):
        self._name = zope.security.proxy.getObject(self.context).addField(obj)


class BaseForm(object):

    def redirect_to_next_url(self, *args):
        # redirect to the entity
        self.request.response.redirect(self.request.getURL(2))


class EditForm(BaseForm, icemac.addressbook.browser.base.GroupEditForm):

    interface = icemac.addressbook.interfaces.IField
    groups = (icemac.addressbook.browser.metadata.MetadataGroup,)


class DeleteForm(BaseForm, icemac.addressbook.browser.base.BaseDeleteForm):

    label = _(
        u'Caution: When you delete this field, possibly data will get lost. '
        u'Namely the data which was entered into this field when it was '
        u'displayed in a form of an object.')
    interface = icemac.addressbook.interfaces.IField
    field_names = ('type', 'title', 'notes')

    def _do_delete(self):
        # We need the name of the entity from the url here to
        # unregister the adapter.
        path = urlparse.urlsplit(self.request.getURL()).path
        entity_name = path.split('/')[-3]
        entity = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntity, name=entity_name)
        sm = zope.site.hooks.getSiteManager()
        sm.unregisterAdapter(
            provided=icemac.addressbook.interfaces.IField,
            required=(icemac.addressbook.interfaces.IEntity, entity.interface),
            name=self.context.__name__)
        return super(DeleteForm, self)._do_delete()



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
# $Id: test_export.py 1369 2011-11-03 19:20:18Z icemac $

import icemac.addressbook.export.base
import icemac.addressbook.export.interfaces
import icemac.addressbook.export.xls.simple
import unittest
import zope.interface.verify


class TestInterfaces(unittest.TestCase):

    def test_BaseExport(self):
        zope.interface.verify.verifyObject(
            icemac.addressbook.export.interfaces.IExporter,
            icemac.addressbook.export.base.BaseExporter([], None))

    def test_XLSExport(self):
        zope.interface.verify.verifyObject(
            icemac.addressbook.export.interfaces.IExporter,
            icemac.addressbook.export.xls.simple.XLSExport([], None))

    def test_XLSDefaultsExport(self):
        zope.interface.verify.verifyObject(
            icemac.addressbook.export.interfaces.IExporter,
            icemac.addressbook.export.xls.simple.DefaultsExport([], None))

    def test_XLSCompleteExport(self):
        zope.interface.verify.verifyObject(
            icemac.addressbook.export.interfaces.IExporter,
            icemac.addressbook.export.xls.simple.CompleteExport([], None))


class Test_export(unittest.TestCase):
    """Test export method of BaseExporter to get better test coverage."""

    def test_export(self):
        exporter = icemac.addressbook.export.base.BaseExporter(None)
        self.assertRaises(NotImplementedError, exporter.export)



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2011 Michael Howitz
# See also LICENSE.txt
# $Id: testing.py 1425 2011-12-05 21:08:55Z icemac $

import gocept.selenium.wsgi
import icemac.addressbook.address
import icemac.addressbook.addressbook
import icemac.addressbook.file.file
import icemac.addressbook.interfaces
import icemac.addressbook.keyword
import icemac.addressbook.person
import icemac.addressbook.principals.principals
import icemac.addressbook.principals.sources
import icemac.addressbook.startup
import icemac.addressbook.utils
import inspect
import lxml.etree
import os
import os.path
import plone.testing
import plone.testing.zca
import plone.testing.zodb
import re
import tempfile
import transaction
import z3c.etestbrowser.wsgi
import zope.annotation.attribute
import zope.app.publication.httpfactory
import zope.app.publication.zopepublication
import zope.app.wsgi.testlayer
import zope.component
import zope.component.hooks
import zope.event
import zope.lifecycleevent
import zope.processlifetime
import zope.site.hooks
import zope.testbrowser.browser
import zope.testbrowser.interfaces
import zope.testbrowser.wsgi
import zope.testing.cleanup
import zope.testing.renormalizing
import zope.testrunner.layer


if os.environ.get('ZOPETESTINGDOCTEST'):  # pragma: no cover
    from zope.testing import doctest
else:
    import doctest


class _AddressBookUnitTests(plone.testing.Layer):
    """Layer for gathering addressbook unit tests."""
    defaultBases = (zope.testrunner.layer.UnitTests,)

    def setUp(self):
        # Some classes use gocept.reference and store default values for
        # which annotations are needed.
        zope.component.provideAdapter(
            zope.annotation.attribute.AttributeAnnotations)

    def tearDown(self):
        zope.testing.cleanup.tearDown()

ADDRESS_BOOK_UNITTESTS = _AddressBookUnitTests(name='AddressBookUnitTests')


ZCML_LAYER = plone.testing.zca.ZCMLSandbox(
    name="AddressBookZCML", filename="ftesting.zcml", module=__name__,
    package=icemac.addressbook)


class _ZODBLayer(plone.testing.zodb.EmptyZODB):
    """Layer which sets up ZODB to be useable for Zope 3."""
    defaultBases = (ZCML_LAYER,)

    def setUp(self):
        super(_ZODBLayer, self).setUp()
        zope.event.notify(zope.processlifetime.DatabaseOpened(self['zodbDB']))
        transaction.commit()

    def testSetUp(self):
        pass

    def testTearDown(self):
        pass

ZODB_LAYER = _ZODBLayer(name='ZODBLayer')


def setUpStackedDemoStorage(self, name):
    self['zodbDB'] = plone.testing.zodb.stackDemoStorage(
        self['zodbDB'], name=name)


def createZODBConnection(zodbDB):
    connection = zodbDB.open()
    zodbRoot = connection.root()
    rootFolder = zodbRoot[
        zope.app.publication.zopepublication.ZopePublication.root_name]
    return connection, zodbRoot, rootFolder


def setUpZODBConnection(self):
    self['zodbConnection'], self['zodbRoot'], self['rootFolder'] = (
        createZODBConnection(self['zodbDB']))
    transaction.begin()


def tearDownZODBConnection(self):
    del self['rootFolder']
    transaction.abort()
    self['zodbConnection'].close()
    del self['zodbConnection']
    del self['zodbRoot']


def tearDownStackedDemoStorage(self):
    self['zodbDB'].close()
    del self['zodbDB']


class _ZODBTestLayer(plone.testing.Layer):
    """Layer which opens the ZODB for each test."""
    defaultBases = (ZODB_LAYER,)

    def testSetUp(self):
        setUpZODBConnection(self)

    def testTearDown(self):
        tearDownZODBConnection(self)

FUNCTIONAL_LAYER = _ZODBTestLayer(name='ZODBTestLayer')


class _ZODBIsolatedTestLayer(plone.testing.Layer):
    """Layer which puts a DemoStorage on ZODB for each test."""
    defaultBases = (ZODB_LAYER,)

    def testSetUp(self):
        setUpStackedDemoStorage(self, self.__name__)
        setUpZODBConnection(self)

    def testTearDown(self):
        tearDownStackedDemoStorage(self)
        tearDownZODBConnection(self)

ZODB_ISOLATED_TEST_LAYER = _ZODBIsolatedTestLayer(name='ZODBIsolatedTestLayer')


class WSGILayer(plone.testing.Layer):
    """Layer which sets up a WSGI stack."""
    defaultBases = (ZODB_ISOLATED_TEST_LAYER,)

    def get_wsgi_pipeline(self):
        return icemac.addressbook.startup.application_factory(
            {}, db=self['zodbDB'])

    def setUp(self):
        self['wsgi_app'] = self.get_wsgi_pipeline()

    def tearDown(self):
        del self['wsgi_app']

    def testSetUp(self):
        # The layer has to store the database at layer set up (depending
        # layers require this) but we get a new database at test set up, so
        # we have to set the right ZODB here:
        self['wsgi_app'].requestFactory = (
            zope.app.publication.httpfactory.HTTPPublicationRequestFactory(
                self['zodbDB']))

WSGI_LAYER = WSGILayer()


class _WSGITestBrowserLayer(zope.testbrowser.wsgi.Layer,
                            plone.testing.Layer):
    """Layer for zope.testbrowser.wsgi tests."""
    defaultBases = (WSGI_LAYER,)

    def make_wsgi_app(self):
        def getRootFolder():
            return self['rootFolder']
        # We could use zope.app.wsgi.testlayer.BrowserLayer but it extends
        # ZODBLayer which we do not want as a base class, so we have to
        # duplicate its middleware stack here:
        return zope.testbrowser.wsgi.AuthorizationMiddleware(
            zope.app.wsgi.testlayer.TransactionMiddleware(
                getRootFolder, self['wsgi_app']))

WSGI_TEST_BROWSER_LAYER = _WSGITestBrowserLayer(name='WSGITestBrowserLayer')


class GoceptSeleniumPloneTestingIntegrationLayer(gocept.selenium.wsgi.Layer,
                                                 plone.testing.Layer):
    """Layer which integrates gocept.selenium with plone.testing."""

    defaultBases = (WSGI_LAYER, )

    def __init__(self, *args, **kw):
        # WSGI application is set up in base layers so we cannot access it
        # here yet:
        gocept.selenium.wsgi.Layer.__init__(self, application=None)
        plone.testing.Layer.__init__(self, *args, **kw)

    def setup_wsgi_stack(self, app):
        return self['wsgi_app']

SELENIUM_LAYER = GoceptSeleniumPloneTestingIntegrationLayer()


def setUpAddressBook(self):
    self.old_site = zope.site.hooks.getSite()
    conn, rootObj, rootFolder = createZODBConnection(self['zodbDB'])
    addressbook = create_addressbook(parent=rootFolder)
    zope.site.hooks.setSite(addressbook)
    transaction.commit()
    conn.close()
    return addressbook


class _AddressBookFunctionalLayer(plone.testing.Layer):
    "Layer where the address book gets created in the layer set up."
    defaultBases = (ZODB_LAYER,)

    def setUp(self):
        setUpStackedDemoStorage(self, 'AddressBookFunctionalTestCase')
        self['addressbook'] = setUpAddressBook(self)

    def tearDown(self):
        zope.site.hooks.setSite(self.old_site)
        del self['addressbook']
        tearDownStackedDemoStorage(self)

    def testSetUp(self):
        setUpZODBConnection(self)
        zope.site.hooks.setSite(self['addressbook'])

    def testTearDown(self):
        tearDownZODBConnection(self)

ADDRESS_BOOK_FUNCTIONAL_LAYER = _AddressBookFunctionalLayer(
    name='AddressBookFunctionalLayer')


# Layer to use ADDRESS_BOOK_FUNCTIONAL_LAYER with testbrowser:
WSGI_ADDRESS_BOOK_FUNCTIONAL_LAYER = _WSGITestBrowserLayer(
    bases=[WSGILayer(bases=[ZODB_ISOLATED_TEST_LAYER,
                            ADDRESS_BOOK_FUNCTIONAL_LAYER],
                     name='WSGIAddressBookFunctionalLayer')],
    name='TestBrowserAddressBookFunctionalLayer')


def DocFileSuite(*paths, **kw):
    """Project specific DocFileSuite."""
    kw['optionflags'] = (kw.get('optionflags', 0) |
                         doctest.ELLIPSIS |
                         doctest.NORMALIZE_WHITESPACE)
    layer = kw.pop('layer')
    globs = kw.setdefault('globs', {})
    globs['layer'] = layer
    if 'checker' not in kw:
        kw['checker'] = zope.testing.renormalizing.RENormalizing([
            (re.compile(r'[0-9]{2}/[0-9]{2}/[0-9]{2} [0-9]{2}:[0-9]{2}'),
             '<DATETIME>')
            ])
    suite = doctest.DocFileSuite(*paths, **kw)
    suite.layer = layer
    return suite


def FunctionalDocFileSuite(*paths, **kw):
    """DocFileSuite on FUNCTIONAL_LAYER."""
    return DocFileSuite(layer=FUNCTIONAL_LAYER, *paths, **kw)


def TestBrowserDocFileSuite(*paths, **kw):
    """DocFileSuite on WSGI_TEST_BROWSER_LAYER."""
    return DocFileSuite(layer=WSGI_TEST_BROWSER_LAYER, *paths, **kw)


# XXX see https://sourceforge.net/tracker/?func=detail&aid=3381282&group_id=273840&atid=2319598
def _get_control_names(interface, browser=None, form=None):
    """Get a sorted list of names of controls providing the given interface.

    Whether the browser was given as argument the form of the browser is used.
    """
    if browser is not None:
        form = browser.getForm()
    else:
        browser = form.browser
    names = []
    for ctrl in form.mech_form.controls:
        control = zope.testbrowser.browser.controlFactory(ctrl, form, browser)
        if interface.providedBy(control):
            names.append(control.name)
    return sorted(names)


def get_submit_control_names(browser=None, form=None):
    """Get a list of the names of the submit controls in the form.

    Whether the browser was given as argument the form of the browser is used.
    """
    return _get_control_names(
        zope.testbrowser.interfaces.ISubmitControl, browser, form)


def get_all_control_names(browser=None, form=None):
    """Get a list of all names of the controls in the form.

    Whether the browser was given as argument the form of the browser is used.
    """
    return _get_control_names(
        zope.testbrowser.interfaces.IControl, browser, form)


def get_messages(browser):
    """Return the info messages displayed in browser.

    Returns string when there is exactly one message, list otherwise."""
    if not isinstance(browser, z3c.etestbrowser.wsgi.ExtendedTestBrowser):
        raise ValueError(
            'browser must be z3c.etestbrowser.wsgi.ExtendedTestBrowser')
    return [x.text
            for x in browser.etree.xpath('//div[@id="info-messages"]/ul/li')]


def in_out_widget_select(browser, control_name, select_controls):
    """Function to add a selection to an in-out-widget.

    browser ... testbrowser instance
    control_name ... name of the in-out-control, something like
                     form.widgets.columns
    select_controls ... list of control instances (item controls of in or out
                        list) which should be selected
    """
    form = browser.getForm()
    for control in select_controls:
        form.mech_form.new_control(
            type='hidden',
            name='%s:list' % control_name,
            attrs=dict(value=control.optionValue))


def write_temp_file(content, suffix):
    "Write `content` to a temporary file and return file handle and file name."
    fd, filename = tempfile.mkstemp(suffix=suffix)
    os.write(fd, content)
    os.close(fd)
    return file(filename, 'r'), os.path.basename(filename)


# List of known users which are able to login using basic auth:
USERNAME_PASSWORD_MAP = dict(mgr='mgrpw', editor='editor', visitor='visitor')


class Browser(z3c.etestbrowser.wsgi.ExtendedTestBrowser):
    """Customized browser which provides login."""

    def login(self, username):
        """Login a user using basic auth."""
        self.addHeader('Authorization', 'Basic %s:%s' %
                       (username, USERNAME_PASSWORD_MAP[username]))

    def etree_to_list(self, etree):
        """"Convert an etree into a list (lines without leading whitespace.)"""
        return [x.strip()
                for x in lxml.etree.tostring(etree).split('\n')
                if x.strip()]

    get_messages = get_messages


### Helper functions to create objects in the database ###


def create_addressbook(name='ab', title=u'test address book', parent=None):
    """Create an address book.

    When parent is `None`, it gets created in the root folder of the data base.

    """
    ab = icemac.addressbook.utils.create_obj(
        icemac.addressbook.addressbook.AddressBook, title=title)
    if parent is None:
        frame = inspect.currentframe()
        try:
            parent = frame.f_back.f_globals['layer']['rootFolder']
        finally:
            del frame
    parent[name] = ab
    return ab


@icemac.addressbook.utils.set_site
def create_keyword(title, return_obj=True):
    """Create a new keyword."""
    parent = zope.component.hooks.getSite().keywords
    name = icemac.addressbook.utils.create_and_add(
        parent, icemac.addressbook.keyword.Keyword, title=title)
    if return_obj:
        return parent[name]


@icemac.addressbook.utils.set_site
def create_person(parent, last_name, return_obj=True, **kw):
    """Create a new person in the address book.

    Caution: The created person is not editable trough the UI, as it
    is missing the default entries. See also `create_full_person`.
    """
    kw['last_name'] = last_name
    name = icemac.addressbook.utils.create_and_add(
        parent, icemac.addressbook.person.Person, **kw)
    if return_obj:
        return parent[name]


@icemac.addressbook.utils.set_site
def create_full_person(parent, last_name, return_obj=True, **kw):
    """Create a new person in the address book with all default entries.

    The created person is editable through the UI.
    """
    person = create_person(parent, parent, last_name, return_obj=True, **kw)
    create_postal_address(parent, person)
    create_email_address(parent, person)
    create_home_page_address(parent, person)
    create_phone_number(parent, person)
    if return_obj:
        return person


@icemac.addressbook.utils.set_site
def create_postal_address(person, set_as_default=True, return_obj=True, **kw):
    name = icemac.addressbook.utils.create_and_add(
        person, icemac.addressbook.address.PostalAddress, **kw)
    address = person[name]
    if set_as_default:
        person.default_postal_address = address
    if return_obj:
        return address


@icemac.addressbook.utils.set_site
def create_email_address(person, set_as_default=True, return_obj=True, **kw):
    name = icemac.addressbook.utils.create_and_add(
        person, icemac.addressbook.address.EMailAddress, **kw)
    address = person[name]
    if set_as_default:
        person.default_email_address = address
    if return_obj:
        return address


@icemac.addressbook.utils.set_site
def create_home_page_address(
    person, set_as_default=True, return_obj=True, **kw):
    name = icemac.addressbook.utils.create_and_add(
        person, icemac.addressbook.address.HomePageAddress, **kw)
    address = person[name]
    if set_as_default:
        person.default_home_page_address = address
    if return_obj:
        return address


@icemac.addressbook.utils.set_site
def create_phone_number(person, set_as_default=True, return_obj=True, **kw):
    name = icemac.addressbook.utils.create_and_add(
        person, icemac.addressbook.address.PhoneNumber, **kw)
    number = person[name]
    if set_as_default:
        person.default_phone_number = number
    if return_obj:
        return number


@icemac.addressbook.utils.set_site
def create_file(person, return_obj=True, **kw):
    "Create a file object inside the `person`."
    name = icemac.addressbook.utils.create_and_add(
        person, icemac.addressbook.file.file.File, **kw)
    if return_obj:
        return person[name]


@icemac.addressbook.utils.set_site
def create(
    parent, entity_name, return_obj=False, set_as_default=False, *args, **kw):
    """Create an object using an entity.

    entity_name ... IEntity.class_name

    """
    entity = zope.component.getUtility(
        icemac.addressbook.interfaces.IEntity, name=entity_name)
    name = icemac.addressbook.utils.create_and_add(
        parent, entity.getClass(), *args)
    obj = parent[name]
    for key, value in kw.items():
        field = entity.getField(key)
        context = field.interface(obj)
        field.set(context, value)
    if set_as_default:
        field = icemac.addressbook.person.get_default_field(entity.interface)
        field.set(parent, obj)
    zope.lifecycleevent.modified(obj)
    if return_obj:
        return obj


@icemac.addressbook.utils.set_site
def create_user(ab, first_name, last_name, email, password, roles, **kw):
    person = create_person(
        ab, ab, last_name, first_name=first_name, **kw)
    create_email_address(ab, person, email=email)

    role_factory = icemac.addressbook.principals.sources.role_source.factory
    role_values = role_factory.getValues()
    selected_roles = []
    for role_title in roles:
        for candidate in role_values:
            if role_factory.getTitle(candidate) == role_title:
                selected_roles.append(candidate)
                break
    icemac.addressbook.utils.create_and_add(
        ab.principals, icemac.addressbook.principals.principals.Principal,
        person=person, password=password, roles=selected_roles)


@icemac.addressbook.utils.set_site
def create_field(entity_name, type, title, **kw):
    """Create a user defined field for an entity.

    entity_name ... IEntity.class_name
    type ... see values of .sources.FieldTypeSource

    Returns the name of the created field.

    To create values for a Choice field use: values=<list of values>

    """
    field = icemac.addressbook.utils.create_obj(
        icemac.addressbook.entities.Field, type=type, title=title, **kw)
    entity = zope.component.getUtility(
        icemac.addressbook.interfaces.IEntity, name=entity_name)
    return entity.addField(field)



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
# $Id: tests.py 1265 2011-03-21 16:58:03Z icemac $

import gocept.reference.verify
import icemac.addressbook.principals.interfaces
import icemac.addressbook.principals.principals
import icemac.addressbook.testing
import unittest
import zope.interface.verify


class TestInterfaces(unittest.TestCase):
    layer = icemac.addressbook.testing.ADDRESS_BOOK_UNITTESTS

    def test_principal(self):
        principal = icemac.addressbook.principals.principals.Principal()
        # need to call created event handler here, because person
        # attribute is a descriptor wrapping the one verifyObject
        # expects.
        icemac.addressbook.principals.principals.created(principal, None)
        gocept.reference.verify.verifyObject(
            icemac.addressbook.principals.interfaces.IPrincipal, principal)
        zope.interface.verify.verifyObject(
            icemac.addressbook.principals.interfaces.IPasswordFields,
            principal)
        zope.interface.verify.verifyObject(
            icemac.addressbook.principals.interfaces.IRoles, principal)



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
import icemac.addressbook.browser.base


class BaseExport(object):
    """Base class for exporters.

    Subclasses only have to set the `exporter_class` which should implement
    the interface `icemac.addressbook.export.interfaces.IExporter`.

    """
    exporter_class = NotImplemented

    def __call__(self):
        session = icemac.addressbook.browser.base.get_session(self.request)
        persons = [self.context[id] for id in session['person_ids']]
        exporter = self.exporter_class(persons, self.request)

        self.request.response.setHeader('Content-Type', exporter.mime_type)
        self.request.response.setHeader(
            'Content-Disposition',
            'attachment; filename=addressbook_export.%s' % (
                exporter.file_extension))

        return exporter.export()



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
# $Id: i18n.py 1229 2011-01-29 17:07:16Z icemac $

"""I18n message factory for icemac.addressbook. """

import zope.i18nmessageid

_ = MessageFactory = zope.i18nmessageid.MessageFactory('icemac.addressbook')



import icemac.addressbook.testing
import unittest2 as unittest
import mock
import plone.testing


class MailToLayer(plone.testing.Layer):

    defaultBases = [icemac.addressbook.testing.WSGI_TEST_BROWSER_LAYER]

    def setUp(self):
        from icemac.addressbook.testing import (
            create_person, create_full_person, create_email_address,
            createZODBConnection, setUpStackedDemoStorage, setUpAddressBook)
        import transaction
        setUpStackedDemoStorage(self, 'SearchLayer')
        ab = setUpAddressBook(self)
        setupZODBConn, rootObj, rootFolder = createZODBConnection(
            self['zodbDB'])

        kw = set([icemac.addressbook.testing.create_keyword(ab, u'mail-me')])
        # No EMailAddress object
        self['p0'] = create_person(ab, ab, u'No Mail', keywords=kw)
        self['p1'] = create_person(ab, ab, u'Mail', keywords=kw)
        create_email_address(ab, self['p1'], email=u'icemac@example.net')
        self['p2'] = create_person(ab, ab, u'Mail Too', keywords=kw)
        create_email_address(ab, self['p2'], email=u'mail@example.com')
        self['p3'] = create_person(ab, ab, u'Double Mail', keywords=kw)
        # Dulicate email address
        create_email_address(ab, self['p3'], email=u'mail@example.com')
        self['p4'] = create_full_person(ab, ab, u'Other Mail', keywords=kw)
        # default email address is None
        create_email_address(
            ab, self['p4'], email=u'other@example.com', set_as_default=False)
        transaction.commit()
        setupZODBConn.close()

    def tearDown(self):
        del self['p0']
        del self['p1']
        del self['p2']
        del self['p3']
        del self['p4']
        icemac.addressbook.testing.tearDownStackedDemoStorage(self)

MAILTO_LAYER = MailToLayer()


class MailToTest(unittest.TestCase):
    """Testing .mailto.MailTo."""

    layer = MAILTO_LAYER

    def get_view(self):
        from .mailto import MailTo
        view = MailTo()
        view.request = mock.Mock()
        return view

    @mock.patch('icemac.addressbook.browser.base.get_session')
    def test_persons_returns_persons_for_ids_in_session(self, session):
        from icemac.addressbook.testing import (
            create_person, create_full_person, create_email_address)
        persons = [self.layer[x] for x in 'p1 p2'.split()]
        session.return_value = dict(person_ids=[x.__name__ for x in persons])
        self.assertEqual(persons, list(self.get_view().get_persons()))

    get_persons = ('icemac.addressbook.browser.search.result.handler.mailto.'
                   'mailto.MailTo.get_persons')

    @mock.patch(get_persons)
    def test_unique_mail_addresses_returns_sorted_unique_email_addresses(
            self, get_persons):
        get_persons.return_value = [
            self.layer[x] for x in 'p0 p1 p2 p3 p4'.split()]

        self.assertEqual([u'icemac@example.net', u'mail@example.com'],
                         self.get_view().unique_mail_addresses)

    def test_view_displays_mail_adresses_of_selected_persons_as_link(self):
        from icemac.addressbook.browser.testing import (
            search_for_persons_with_keyword_search_using_browser)
        browser = search_for_persons_with_keyword_search_using_browser(
            'mail-me', 'visitor')
        browser.handleErrors = False
        browser.getControl('Apply on selected persons').displayValue = [
            'E-Mail']
        browser.getControl(name='form.buttons.apply').click()
        file('response.html', 'w').write(browser.contents)
        self.assertEqual(
            ['<div id="mailto">',
             '<h2>Send an e-mail</h2>',
             '<a href="mailto:?bcc=icemac@example.net,mail@example.com">'
             'Click here to open your e-mail client</a>',
             '</div>'],
            browser.etree_to_list(
                browser.etree.xpath('//div[@id="mailto"]')[0]))



import unittest2


class MockEntitiesAndOrder(object):
    """Mock wich implements parts of IEntities and IEntityOrder."""

    def __init__(self):
        import icemac.addressbook.address
        self.order = {
            icemac.addressbook.address.postal_address_entity: 0,
            icemac.addressbook.address.phone_number_entity: 10,
            icemac.addressbook.address.e_mail_address_entity: 20,
            icemac.addressbook.address.home_page_address_entity: 30,
            }

    def get(self, entity):
        return self.order[entity]

    def getEntities(self, **kw):
        return self.order.keys()


class PersonDefaultsEntityTest(unittest2.TestCase):

    def setUp(self):
        import icemac.addressbook.interfaces
        import zope.component.testing

        zope.component.testing.setUp()
        mock = MockEntitiesAndOrder()
        zope.component.provideUtility(
            mock, icemac.addressbook.interfaces.IEntityOrder)
        zope.component.provideUtility(
            mock, icemac.addressbook.interfaces.IEntities)

    def tearDown(self):
        import zope.component.testing
        zope.component.testing.tearDown()

    def change_sortorder(self):
        import icemac.addressbook.address
        import icemac.addressbook.interfaces
        import zope.component

        # Switching first two entries:
        entity_order = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntityOrder)
        entity_order.order[
            icemac.addressbook.address.postal_address_entity] = 15

    def callFUT(self, sorted=True):
        import icemac.addressbook.person
        pde = icemac.addressbook.person.PersonDefaultsEntity(
            None, icemac.addressbook.interfaces.IPersonDefaults, None)
        return [x[0] for x in pde.getRawFields(sorted=sorted)]

    def test_default_sortorder(self):
        self.assertEqual(
            ['default_postal_address', 'default_phone_number',
             'default_email_address', 'default_home_page_address'],
            self.callFUT())

    def test_changed_sortorder(self):
        self.change_sortorder()
        self.assertEqual(
            ['default_phone_number', 'default_postal_address',
             'default_email_address', 'default_home_page_address'],
            self.callFUT())

    def test_changed_sortorder_not_sorted(self):
        self.change_sortorder()
        self.assertEqual(
            ['default_postal_address', 'default_phone_number',
             'default_email_address', 'default_home_page_address'],
            self.callFUT(sorted=False))


class TestPersonName(unittest2.TestCase):

    def callFUT(self, first_name=None, last_name=None):
        import icemac.addressbook.person
        person = icemac.addressbook.person.Person()
        if first_name:
            person.first_name = first_name
        if last_name:
            person.last_name = last_name
        return icemac.addressbook.person.PersonName(person).get_name()

    def test_get_name_returns_last_name_and_first_name_when_existing(self):
        self.assertEqual(u'Bernd Tester', self.callFUT(u'Bernd', u'Tester'))

    def test_get_name_returns_last_name_when_first_name_not_existing(self):
        self.assertEqual(u'Tester', self.callFUT(last_name=u'Tester'))

    def test_get_name_returns_first_name_when_last_name_not_existing(self):
        self.assertEqual(u'Berns', self.callFUT(first_name=u'Berns'))

    def test_get_name_returns_empty_string_when_no_name_exists(self):
        self.assertEqual(u'', self.callFUT())



# -*- coding: utf-8 -*-
# Copyright (c) 2011 Michael Howitz
# See also LICENSE.txt
from icemac.addressbook.browser.testing import (
    search_for_persons_with_keyword_search_using_browser)


def select_persons_with_keyword_for_update(keyword):
    """Returns a browser where the persons with the given keyword are
    selected for the update handler.

    """
    browser = search_for_persons_with_keyword_search_using_browser(keyword)
    browser.getControl('Apply on selected persons').displayValue = ['Update']
    browser.getControl(name='form.buttons.apply').click()
    return browser



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
# $Id: test_doctests.py 1370 2011-11-03 19:36:53Z icemac $

import icemac.addressbook.testing
import unittest


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(icemac.addressbook.testing.FunctionalDocFileSuite(
        # Caution: none of these tests can run as unittest!
        'adapter.txt',
        'address.txt',
        'addressbook.txt',
        'person.txt',
        ))
    suite.addTest(icemac.addressbook.testing.TestBrowserDocFileSuite(
        'testing.txt',
        ))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2011 Michael Howitz
# See also LICENSE.txt

import zope.interface


class IDontReuseNames(zope.interface.Interface):
    """Marker interface for container those names are never reused.

    Even when contained objects get deleted their names are _not_ reused.
    """


class INameSuffix(zope.interface.Interface):
    """Suffix to make name in container unique."""

    def __iadd__(value):
        "Increment the suffix by `value`."

    def __unicode__():
        "Convert suffix to unicode."



import unittest


class TestSearchResultHandler(unittest.TestCase):
    """Testing .manager.SearchResultHandler."""

    def test_two_SearchResultHandlers_are_equal_when_viewName_is_equal(self):
        from icemac.addressbook.browser.search.result.handler.manager import (
            SearchResultHandler)
        h1 = SearchResultHandler(None, None, None, None)
        h1.viewName = '@@asdf.html'
        h2 = SearchResultHandler(None, None, None, None)
        h2.viewName = '@@asdf.html'
        self.assertEqual(h1, h2)

    def test_two_SearchResultHandlers_are_not_equal_when_viewName_is_unequal(
        self):
        from icemac.addressbook.browser.search.result.handler.manager import (
            SearchResultHandler)
        h1 = SearchResultHandler(None, None, None, None)
        h1.viewName = '@@foo.html'
        h2 = SearchResultHandler(None, None, None, None)
        h2.viewName = '@@bar.html'
        self.assertFalse(h1 == h2)

    def test_SearchResultHandler_is_not_equal_to_any_thing_else(self):
        from icemac.addressbook.browser.search.result.handler.manager import (
            SearchResultHandler)
        h1 = SearchResultHandler(None, None, None, None)
        self.assertFalse(h1 == object())



# -*- coding: utf-8 -*-
# Copyright (c) 2011 Michael Howitz
# See also LICENSE.txt

import icemac.addressbook.testing
import unittest


class SerializableObject(object):

    text = u'asdf'
    number1 = 2411
    number2 = 1234.6789
    bytes = 'foobar'


class TestFieldSerializer(unittest.TestCase):

    def callFUT(self, field_class_name, field_name, serializer_class_name,
                expected_result):
        import zope.schema
        import icemac.addressbook.catalog.serializer

        field = getattr(zope.schema, field_class_name)(title=u'')
        field.__name__ = field_name
        serializer = getattr(icemac.addressbook.catalog.serializer,
                             serializer_class_name)
        result = serializer(field, SerializableObject())
        self.assertTrue(isinstance(result, unicode))
        self.assertEqual(expected_result, result)

    def test_serializer_returns_field_value_of_textline_as_unicode(self):
        self.callFUT('TextLine', 'text', 'FieldSerializer', u'asdf')

    def test_serializer_returns_field_value_of_text_as_unicode(self):
        self.callFUT('Text', 'text', 'FieldSerializer', u'asdf')

    def test_serializer_returns_field_value_of_int_as_unicode(self):
        self.callFUT('Int', 'number1', 'FieldSerializer', u'2411')

    def test_serializer_returns_field_value_of_float_as_unicode(self):
        self.callFUT('Float', 'number2', 'FieldSerializer', u'1234.6789')

    def test_noserializer_returns_empty_unicode(self):
        self.callFUT('Bytes', 'bytes', 'FieldNoSerializer', u'')


class TestObjectSerializer(unittest.TestCase):

    layer = icemac.addressbook.testing.FUNCTIONAL_LAYER

    def test_serializer_serializes_all_field_values(self):
        import gocept.country.db
        import icemac.addressbook.address

        address = icemac.addressbook.address.PostalAddress()
        address.city = u'Dunkelhausen'
        address.country = gocept.country.db.Country('DE')



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2011 Michael Howitz
# See also LICENSE.txt

import icemac.addressbook.testing


def test_suite():
    return icemac.addressbook.testing.TestBrowserDocFileSuite(
        'person-list-batching.txt',
        'person-list-userfields.txt',
        'person-list.txt',
        'preferences.txt',
        'sortorder.txt',
        package="icemac.addressbook.preferences")



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
# $Id: simple.py 1370 2011-11-03 19:36:53Z icemac $

import icemac.addressbook.browser.search.result.base


class PersonTable(
    icemac.addressbook.browser.search.result.base.BasePersonTable):
    """Result table to register name column on it (via ZCML)."""

    sortOn = u'table-name-1'


class ExportForm(
    icemac.addressbook.browser.search.result.base.BaseSearchResultForm):
    "Export form showing a table containing the names of the found persons."

    table_class = PersonTable



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
# $Id: xls.py 1369 2011-11-03 19:20:18Z icemac $

import icemac.addressbook.browser.search.result.handler.export.base
import icemac.addressbook.export.xls.simple


class DefaultsExport(
    icemac.addressbook.browser.search.result.handler.export.base.BaseExport):
    """Exporter for default data and addresses."""

    exporter_class = icemac.addressbook.export.xls.simple.DefaultsExport


class CompleteExport(
    icemac.addressbook.browser.search.result.handler.export.base.BaseExport):
    """Exporter for all data and addresses."""

    exporter_class = icemac.addressbook.export.xls.simple.CompleteExport



# -*- coding: utf-8 -*-
# Copyright (c) 2011 Michael Howitz
# See also LICENSE.txt

import datetime
import icemac.addressbook.testing
import plone.testing
from icemac.addressbook.testing import (
    create_addressbook, create_person, create_full_person, create_keyword,
    create_user)
import icemac.addressbook.testing
import transaction


class _SearchLayer(plone.testing.Layer):
    """Layer wich creates base data for searches."""
    defaultBases = (icemac.addressbook.testing.WSGI_LAYER,)

    def setUp(self):
        icemac.addressbook.testing.setUpStackedDemoStorage(self, 'SearchLayer')
        addressbook = icemac.addressbook.testing.setUpAddressBook(self)
        setupZODBConn, rootObj, rootFolder = (
            icemac.addressbook.testing.createZODBConnection(self['zodbDB']))

        friends = self['kw_friends'] = create_keyword(addressbook, u'friends')
        family = self['kw_family'] = create_keyword(addressbook, u'family')
        church = self['kw_church'] = create_keyword(addressbook, u'church')
        self['kw_work'] = create_keyword(addressbook, u'work')
        self['kw_anyone_else'] = create_keyword(addressbook, u'anyone else')
        # person objects are not stored at layer as ZODBIsolatedTestLayer
        # puts a DemoStorage on the storage the persons are created in, so
        # changes on the persons are not visible on the persons stored at
        # the layer.
        create_person(
            addressbook, addressbook, u'Hohmuth', keywords=set([friends]))
        create_full_person(
            addressbook, addressbook, u'Koch', keywords=set([family, church]),
            birth_date=datetime.date(1952, 1, 24), notes=u'father-in-law')
        create_full_person(addressbook, addressbook, u'Velleuer',
                           keywords=set([family, church]))
        create_person(addressbook, addressbook, u'Liebig',
                      keywords=set([church]), notes=u'family')
        create_person(addressbook, addressbook, u'Tester', first_name=u'Liese')
        transaction.commit()
        setupZODBConn.close()

    def tearDown(self):
        del self['kw_family']
        del self['kw_church']
        del self['kw_work']
        del self['kw_anyone_else']
        icemac.addressbook.testing.tearDownStackedDemoStorage(self)

SEARCH_LAYER = _SearchLayer(name='SearchLayer')


WSGI_SEARCH_LAYER = icemac.addressbook.testing._WSGITestBrowserLayer(
    bases=(SEARCH_LAYER,), name='WSGISearchLayer')


def search_for_persons_with_keyword_search_using_browser(keyword, login='mgr'):
    """Searches for all persons with the given keyword.

    Returns the browser.

    """
    browser = icemac.addressbook.testing.Browser()
    browser.login(login)
    browser.open('http://localhost/ab/@@multi_keyword.html')
    browser.getControl('keywords').displayValue = [keyword]
    browser.getControl('Search').click()
    return browser



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
# $Id: interfaces.py 1257 2011-03-05 14:18:16Z icemac $

from icemac.addressbook.i18n import MessageFactory as _
import gocept.country
import gocept.country.db
import gocept.reference.field
import icemac.addressbook.sources
import re
import zope.interface
import zope.schema


PACKAGE_ID = 'icemac.addressbook'
ENTITIES = 'entites_namespace'
FIELD_NS_PREFIX = 'fields-'


class ITitle(zope.interface.Interface):
    """Title of an entity."""

    def __str__():
        """Return the title of the entity."""


class IAddressBook(zope.interface.Interface):
    """An address book."""

    keywords = zope.interface.Attribute(
        u'icemac.addressbook.interfaces.IKeywords')
    principals = zope.interface.Attribute(
        u'zope.pluggableauth.plugins.principalfolder.'
        u'IInternalPrincipalContainer')
    importer = zope.interface.Attribute(
        u'icemac.addressbook.importer.interfaces.IImportContainer')
    entities = zope.interface.Attribute(
        u'icemac.addressbook.interfaces.IEntities')
    orders = zope.interface.Attribute(
        u'icemac.addressbook.interfaces.IOrderStorage')

    title = zope.schema.TextLine(title=_(u'title'))


class IPerson(zope.interface.Interface):
    """A person."""

    first_name = zope.schema.TextLine(title=_(u'first name'), required=False)
    last_name = zope.schema.TextLine(title=_(u'last name'))
    birth_date = zope.schema.Date(title=_(u'birth date'), required=False)
    keywords = gocept.reference.field.Set(
        title=_('keywords'), required=False,
        value_type=zope.schema.Choice(
            title=_('keywords'),
            source=icemac.addressbook.sources.keyword_source))
    notes = zope.schema.Text(title=_(u'notes'), required=False)


class IPersonName(zope.interface.Interface):
    """Name of a person, used for indexing."""

    def get_name():
        """Return first name and last name of the person."""


class IPostalAddress(zope.interface.Interface):
    """A postal address."""

    address_prefix = zope.schema.TextLine(
        title=_(u'address prefix'), required=False,
        description=_(u'e. g. company name or c/o'))
    street = zope.schema.TextLine(title=_(u'street'), required=False)
    city = zope.schema.TextLine(title=_(u'city'), required=False)
    zip = zope.schema.TextLine(title=_(u'zip'), required=False)
    country = zope.schema.Choice(
        title=_(u'country'), source=gocept.country.countries,
        required=False, default=gocept.country.db.Country('DE'))


class IEMailAddress(zope.interface.Interface):
    """An e-mail address."""

    email = zope.schema.TextLine(
        title=_(u'e-mail address'), required=False,
        constraint=re.compile(
            "^[/$!%=+A-Za-z0-9_.-]+@([A-Za-z0-9_\-]+\.)+[A-Za-z]{2,6}$").match)


class IHomePageAddress(zope.interface.Interface):
    """A home page address."""

    url = zope.schema.URI(title=_(u'URL'), required=False)


class IPhoneNumber(zope.interface.Interface):
    """A phone number."""

    number = zope.schema.TextLine(title=_(u'number'), required=False)


class IPersonDefaults(zope.interface.Interface):
    """Default addresses, phone numbers etc."""

    default_postal_address = zope.schema.Choice(
        title=_(u'main postal address'),
        source=icemac.addressbook.sources.ContextByInterfaceSource(
            IPostalAddress))
    default_phone_number = zope.schema.Choice(
        title=_(u'main phone number'),
        source=icemac.addressbook.sources.ContextByInterfaceSource(
            IPhoneNumber))
    default_email_address = zope.schema.Choice(
        title=_(u'main e-mail address'),
        source=icemac.addressbook.sources.ContextByInterfaceSource(
            IEMailAddress))
    default_home_page_address = zope.schema.Choice(
        title=_(u'main home page address'),
        source=icemac.addressbook.sources.ContextByInterfaceSource(
            IHomePageAddress))


class IKeywords(zope.interface.Interface):
    """Collection of keywords."""

    def get_keywords():
        """Get an iterable of the keywords in the collection."""

    def get_keyword_by_title(title, default=None):
        "Get the keyword with the given title or `default`."


class IKeywordTitles(zope.interface.Interface):
    """Collection of keywords titles."""

    def get_titles():
        """Get the titles of the keywords in the collection."""


class IKeyword(zope.interface.Interface):
    """A keyword."""

    title = zope.schema.TextLine(title=_(u'keyword'))


class IEntities(zope.interface.Interface):
    """Entities in the address book."""

    def getEntities(sorted=True):
        """Get an iterable of all known entities.

        When `sorted` is True, order them as defined in IEntityOrder.

        """

    def getMainEntities(sorted=True):
        """Get an iterable of the most important entities.

        When `sorted` is True, order them as defined in IEntityOrder.

        """


class IEntityRead(zope.interface.Interface):
    """Entity in the address book. (Read only part of the interface.)"""

    title = zope.schema.TextLine(
        title=u"title of the entity", required=False)
    interface = zope.schema.InterfaceField(title=u"interface")
    class_name = zope.schema.DottedName(
        title=u"dotted name of the class", required=False)

    name = zope.interface.Attribute(
        "Uniqe name of the entity which only contains letters.")
    tagged_values = zope.interface.Attribute(
        "Dict of tagged values of the entity.")

    def getRawField(field_name):
        """Get a field by its name. (Without any conversion.)"""

    def getField(field_name):
        """Get a zope.schema field by its name.

        When the field is a user defined one it gets converted to a zope.schema
        field.
        """

    def getFieldOrder():
        """Get the ordered names of the fields.

        Caution: This method only returns the names for the fields which are
        known in the field order!
        """

    def getRawFields(sorted=True):
        """Get (name, field) tuples of the schema fields on the entity.

        When `sorted` is true the fields are sorted using the field order.

        Returns static (zope.schema) and user defined (IField) fields.

        """

    def getFields(sorted=True):
        """Get (name, field) tuples of the schema fields on the entity.

        When `sorted` is true the fields are sorted using the field order.

        Converts user defined fields (see IField) into zope.schema fields.

        """

    def getFieldValues(sorted=True):
        """Get list of the schema fields on the entity.

        When `sorted` is true the fields are sorted using the field order.

        Converts user defined fields (see IField) into zope.schema fields.

        """

    def getClass():
        """Get the class object for `self.class_name`."""


class IEntityWrite(zope.interface.Interface):
    """Entity in the address book. (Write part of the interface.)"""

    def addField(field):
        """Add a user defined field to the entity."""

    def setFieldOrder(field_names):
        """Update the order of the fields like in `field_names`.

        field_names ... list of the names of the fields as returned by
                        `getFields`.

        Field names which do not belong to the entity are omitted.
        """


class IEntity(IEntityRead, IEntityWrite):
    """Entity in the address book."""


class IEditableEntity(IEntity):
    """Special entity which is editable.

    This means that new fields can be added and the fields can be sorted.

    """


class IField(zope.interface.Interface):
    """User defined field."""

    __name__ = zope.interface.Attribute('internal field name')

    type = zope.schema.Choice(
        title=_(u'type'),
        source=icemac.addressbook.sources.FieldTypeSource())
    title = zope.schema.TextLine(title=_(u'title'))
    values = zope.schema.List(
        title=_(u'choice field values'), unique=True, required=False,
        value_type=zope.schema.TextLine(title=_(u'value')))
    notes = zope.schema.Text(title=_(u'notes'), required=False)

    @zope.interface.invariant
    def choice_type_needs_values(field):
        if field.type == u'Choice' and not field.values:
            raise zope.interface.Invalid(
                _(u'type "choice" requires at least one field value.'))


class IMayHaveUserFields(zope.interface.Interface):
    """Marker interface: Object may have user defined fields."""


class IUserFieldStorage(zope.interface.Interface):
    """Storage for user defined field values."""


class IOrderStorageRead(zope.interface.Interface):
    """Storage of orders of objects. (read only methods)"""

    def namespaces():
        """Get an iterable of the known namespaces."""

    def get(obj, namespace):
        """Get the index of the object in the list.

        Raises a KeyError when the object is not in the list.
        """

    def isFirst(obj, namespace):
        """Tell whether `obj` is the first object in the list.

        Raises a KeyError when the object is not in the list.
        """

    def isLast(obj, namespace):
        """Tell whether `obj` is the last object in the list.

        Raises a KeyError when the object is not in the list.
        """

    def __iter__(namespace):
        """Iterate over the objects registered for the namespace."""


class IOrderStorageWrite(zope.interface.Interface):
    """Storage of orders of objects. (write methods)"""

    def add(obj, namespace):
        """Add an object to the order for a namespace.

        When the namespace does not exist, it gets created.
        New objects are added at the end of the list.
        When the object is already in the list it is not added again and
        does not change its position.
        """

    def remove(obj, namespace):
        """Remove the object from the order of a namespace.

        Raises a ValueError when the object is not in the list.
        Raises a KeyError when the namespace does not exist.
        """

    def truncate(namespace):
        """Remove all objects from the order of a namespace.

        Does nothing when the namespace is not known.
        """

    def up(obj, namespace, delta=1):
        """Move the object one position up in the list.

        `delta` describes the number of positions to move.

        When it would be moved beyond the beginning of the list a
        ValueError is raised.
        """

    def down(obj, namespace, delta=1):
        """Move the object one position down in the list.

        `delta` describes the number of positions to move.

        When it would be moved beyond the end of the list a ValueError is
        raised.
        """


class IOrderStorage(IOrderStorageRead, IOrderStorageWrite):
    """Storage of orders of objects."""


class IEntityOrder(zope.interface.Interface):
    """Order of entities."""

    def get(entity):
        """Get the index of the entity in the entity order.

        Raises a KeyError when the entity is not known.
        """

    def isFirst(entity):
        """Tell whether `entity` comes first in the entity order.

        Raises a KeyError when the entity is not known.
        """

    def isLast(entity):
        """Tell whether `entity` comes last in the entity order.

        Raises a KeyError when the entity is not known.
        """

    def __iter__():
        """Iterate over the entities sorted by order."""

    def up(entity, delta=1):
        """Move the entity one position up in the entity order.

        `delta` describes the number of positions to move.

        When the entity would be moved beyond the beginning of the entity
        order a ValueError is raised.
        """

    def down(entity, delta=1):
        """Move the entity one position down in the entity order.

        `delta` describes the number of positions to move.

        When it would be moved beyond the end of the entity order a
        ValueError is raised.
        """



# -*- coding: utf-8 -*-
# Copyright (c) 2011 Michael Howitz
# See also LICENSE.txt
import icemac.addressbook.browser.testing
import unittest2 as unittest
import urllib2


class TestOnlyAdminIsAllowedToUseUpdate(unittest.TestCase):

    layer = icemac.addressbook.browser.testing.WSGI_SEARCH_LAYER

    def _get_browser(self, username):
        import icemac.addressbook.testing
        browser = icemac.addressbook.testing.Browser()
        browser.login(username)
        return browser

    def update_handler_is_not_visible_for(self, username):
        browser = self._get_browser(username)
        browser.open('http://localhost/ab/@@multi_keyword.html')
        browser.getControl('keywords').displayValue = ['church']
        browser.getControl('Search').click()
        self.assertEqual(
            ['XLS export main (Exports person data and main addresses resp. '
              'phone numbers.)',
             'XLS export complete (Exports person data and all addresses '
               'resp. phone numbers.)',
             'E-Mail (Creates a link to send e-mails.)'],
            browser.getControl('Apply on selected persons').displayOptions)

    def each_part_of_the_update_wizard_is_not_accessible_for(self, username):
        browser = self._get_browser(username)
        for url in ('http://localhost/ab/@@multi-update',
                    'http://localhost/ab/multi-update/enterValue',
                    'http://localhost/ab/multi-update/checkResult',
                    'http://localhost/ab/@@multi-update-completed'):
            self._assert_access_raises_forbidden(browser, url)

    def _assert_access_raises_forbidden(self, browser, url):
        with self.assertRaises(urllib2.HTTPError) as err:
            browser.open(url)
        self.assertEqual('HTTP Error 403: Forbidden', str(err.exception))

    # Tests

    def test_editor_is_not_able_to_see_update_search_result_handler(self):
        self.update_handler_is_not_visible_for('editor')

    def test_editor_is_not_able_to_access_update_search_result_handler(self):
        # even though he knows the url
        self.each_part_of_the_update_wizard_is_not_accessible_for('editor')

    def test_visitor_is_not_able_to_see_update_search_result_handler(self):
        self.update_handler_is_not_visible_for('visitor')

    def test_visitor_is_not_able_to_access_update_search_result_handler(self):
        # even though he knows the url
        self.each_part_of_the_update_wizard_is_not_accessible_for('visitor')



from __future__ import unicode_literals
from decimal import Decimal
import icemac.addressbook.testing
import unittest


class TestOperators(unittest.TestCase):
    """Testing ..operators.*"""

    layer = icemac.addressbook.testing.ZODB_ISOLATED_TEST_LAYER

    def callOP(self, value1, value2, operator_name):
        import zope.component
        from ..operators import IOperator
        operator = zope.component.getAdapter(
            value1, IOperator, name=operator_name)
        return operator(value2)

    def create_folders(self, quantity):
        """Create some folders needed for keyword tests."""
        from icemac.addressbook.utils import create_and_add
        from zope.container.folder import Folder
        root = self.layer['rootFolder']
        folders = tuple()
        for i in range(quantity):
            folders += (root[create_and_add(root, Folder)], )
        return folders

    def assert_set_op(self, expected, operand1, operator, operand2):
        from gocept.reference.collection import InstrumentedSet
        import zope.component.hooks
        old_site = zope.component.hooks.getSite()
        try:
            zope.component.hooks.setSite(self.layer['rootFolder'])
            self.assertEqual(
                set(expected),
                self.callOP(
                    InstrumentedSet(operand1), set(operand2), operator))
        finally:
            zope.component.hooks.setSite(old_site)

    def test_append_returns_value2_if_value2_is_None(self):
        self.assertEqual('bar', self.callOP(None, u'bar', 'append'))

    def test_append_returns_concatenation_for_unicode(self):
        self.assertEqual('foobar', self.callOP('foo', 'bar', 'append'))

    def test_replace_returns_value2_for_unicodes(self):
        self.assertEqual('bar', self.callOP('foo', 'bar', 'replace'))

    def test_replace_returns_value2_for_ints(self):
        self.assertEqual(2, self.callOP(1, 2, 'replace'))

    def test_prepend_with_None_returns_value2(self):
        self.assertEqual('bar', self.callOP(None, 'bar', 'prepend'))

    def test_prepend_returns_inverse_concatenation_of_append(self):
        self.assertEqual('barfoo', self.callOP('foo', 'bar', 'prepend'))

    def test_remove_all_removes_all_occurrences_of_value2_in_value1(self):
        self.assertEqual('bac', self.callOP('abbaabc', 'ab', 'remove-all'))

    def test_remove_all_does_nothing_on_None(self):
        self.assertEqual(None, self.callOP(None, 'a', 'remove-all'))

    def test_remove_first_removes_only_first_occurrence_of_value2(self):
        self.assertEqual('ddab', self.callOP('abddab', 'ab', 'remove-first'))

    def test_remove_first_does_nothing_on_None(self):
        self.assertEqual(None, self.callOP(None, 'a', 'remove-first'))

    def test_remove_last_removes_only_last_occurrence_of_value2(self):
        self.assertEqual('abdd', self.callOP('abddab', 'ab', 'remove-last'))

    def test_remove_last_does_nothing_on_None(self):
        self.assertEqual(None, self.callOP(None, 'a', 'remove-last'))

    def test_union_returns_union_of_values(self):
        f1, f2, f3 = self.create_folders(3)
        self.assert_set_op([f1, f2, f3], [f1, f2], 'union', [f2, f3])

    def test_difference_returns_difference_of_values(self):
        f1, f2, f3, f4 = self.create_folders(4)
        self.assert_set_op([f1, f3],
                           [f1, f2, f3, f4], 'difference', [f2, f4])

    def test_intersection_returns_intersection_of_values(self):
        f1, f2, f3, f4 = self.create_folders(4)
        self.assert_set_op([f2],
                           [f1, f2, f3], 'intersection', [f2, f4])

    def test_symmetric_difference_returns_symmetric_difference_of_values(self):
        f1, f2, f3 = self.create_folders(3)
        self.assert_set_op([f1, f3],
                           [f1, f2], 'symmetric_difference', [f2, f3])

    def test_add_returns_value2_on_None(self):
        self.assertEqual(2, self.callOP(None, 2, 'add'))

    def test_add_returns_sum_of_value1_and_value2(self):
        self.assertEqual(3, self.callOP(1, 2, 'add'))

    def test_add_returns_sum_of_value1_and_value2_for_decimals(self):
        self.assertEqual(Decimal(3),
                         self.callOP(Decimal(1), Decimal(2), 'add'))

    def test_sub_returns_neg_value2_on_None(self):
        self.assertEqual(-2, self.callOP(None, 2, 'sub'))

    def test_sub_returns_difference_of_value1_and_value2(self):
        self.assertEqual(-1, self.callOP(1, 2, 'sub'))

    def test_sub_returns_difference_of_value1_and_value2_for_decimals(self):
        self.assertEqual(Decimal(-1),
                         self.callOP(Decimal(1), Decimal(2), 'sub'))

    def test_mul_returns_0_on_None(self):
        self.assertEqual(0, self.callOP(None, 2, 'mul'))

    def test_mul_returns_product_of_value1_and_value2(self):
        self.assertEqual(6, self.callOP(3, 2, 'mul'))

    def test_mul_returns_product_of_value1_and_value2_for_decimals(self):
        self.assertEqual(Decimal(6),
                         self.callOP(Decimal(3), Decimal(2), 'mul'))

    def test_div_returns_0_on_None(self):
        self.assertEqual(0, self.callOP(None, 2, 'div'))

    def test_div_returns_whole_number_quotient_of_value1_and_value2(self):
        self.assertEqual(3, self.callOP(10, 3, 'div'))

    def test_div_returns_quotient_of_value1_and_value2(self):
        self.assertEqual(Decimal(10) / Decimal(3),
                         self.callOP(Decimal(10), Decimal(3), 'div'))



import unittest2 as unittest
import icemac.addressbook.testing


class Test_default_attrib_name_to_entity(unittest.TestCase):

    def setUp(self):
        from icemac.addressbook.tests.stubs import setUpStubEntities
        import icemac.addressbook.entities
        import zope.component.testing

        zope.component.testing.setUp()
        setUpStubEntities(self, icemac.addressbook.entities.Entities)

    def tearDown(self):
        import zope.component.testing
        zope.component.testing.tearDown()

    def callFUT(self, name):
        from icemac.addressbook.address import default_attrib_name_to_entity
        return default_attrib_name_to_entity(name)

    def test_unknown_name_raises_ValueError(self):
        self.assertRaises(ValueError, self.callFUT, 'foo')

    def test_known_name_returns_entity(self):
        self.assertEqual(self.duck, self.callFUT('default_duck'))


class Test_normalize_phone_number(unittest.TestCase):

    def callFUT(self, number, country_code="+49"):
        import icemac.addressbook.address
        return icemac.addressbook.address.normalize_phone_number(
            number, country_code)

    def test_normalized_number_is_returned_unchanged(self):
        self.assertEqual('+491234567890', self.callFUT('+491234567890'))

    def test_everything_but_numbers_and_leading_plus_is_removed(self):
        self.assertEqual(
            '+491234567890', self.callFUT('+49 (1234) 5678 - 90X'))

    def test_leading_zero_is_replaced_by_country_code(self):
        self.assertEqual('+491234567891', self.callFUT('01234/5678-91'))

    def test_only_first_zero_is_replaced_by_country_code(self):
        self.assertEqual('+491234507090', self.callFUT('01234/5070-90'))

    def test_first_zero_is_not_replaced_when_country_code_is_empty(self):
        self.assertEqual('01234567891',
                         self.callFUT('01234/5678-91', country_code=''))

    def test_leading_double_zeros_are_replaced_by_plus(self):
        self.assertEqual('+421234567891', self.callFUT('0042-1234/5678-91'))

    def test_only_leading_double_zeros_are_replaced_by_plus(self):
        self.assertEqual('+421234007891', self.callFUT('0042-1234/0078-91'))


class Test_email_address_of_person(unittest.TestCase):
    """Testing ..address.email_address_of_person."""

    layer = icemac.addressbook.testing.ADDRESS_BOOK_FUNCTIONAL_LAYER

    def setUp(self):
        from icemac.addressbook.testing import create_person
        self.ab = self.layer['addressbook']
        self.person = create_person(self.ab, self.ab, u'Tester')

    def test_returns_default_email_address(self):
        from icemac.addressbook.testing import create_email_address
        from icemac.addressbook.interfaces import IEMailAddress
        create_email_address(
            self.ab, self.person, email=u'tester@exmaple.com',
            set_as_default=False)
        create_email_address(
            self.ab, self.person, email=u't@exmaple.net',
            set_as_default=True)
        self.assertEqual(u't@exmaple.net', IEMailAddress(self.person).email)

    def test_raises_exception_if_default_email_address_does_not_exist(self):
        from icemac.addressbook.interfaces import IEMailAddress
        with self.assertRaises(TypeError) as err:
            IEMailAddress(self.person)
        self.assertEqual('Could not adapt', err.exception[0])



import gocept.selenium.wsgi
import icemac.addressbook.testing
import transaction


class TestFieldOrder(gocept.selenium.wsgi.TestCase):

    layer = icemac.addressbook.testing.SELENIUM_LAYER
    level = 2

    def setUp(self):
        super(TestFieldOrder, self).setUp()
        ab = icemac.addressbook.testing.create_addressbook(
            parent=self.layer['rootFolder'])
        icemac.addressbook.testing.create_user(
            ab, ab, u'Selenium', u'Tester', u'sel@example.com', '12345678',
            ['Administrator'])
        transaction.commit()

    def test_field_movement(self):
        sel = self.selenium
        sel.open('http://%s/++skin++AddressBook/ab' % self.selenium.server)
        # Login
        sel.type("login", "sel@example.com")
        sel.type("password", "12345678")
        sel.clickAndWait("SUBMIT")
        # Open Masterdata
        sel.clickAndWait("//ul[@id='main-menu']/li[4]/a/span")
        sel.assertLocation(r'regexp:@@masterdata\.html$')
        # Go to entities
        sel.clickAndWait("//div[@id='content']/ul/li[3]/a/span")
        sel.assertLocation(r'regexp:\+\+attribute\+\+entities$')
        # Open fields of person entity
        sel.clickAndWait("//div[@id='content']/table/tbody/tr[2]/td[4]/a")
        sel.assertLocation(r'regexp:icemac\.addressbook\.person\.Person$')

        # Keywords-Field has a language depenend title, we store it here for
        # the assertion of the movement later on
        fieldtitle = sel.getText("//tr[4]/td[1]")

        # The Drag'n'drop commands of selenium don't work so we have to
        # emulate them:
        sel.mouseDownAt("//tr[4]/td[1]", "5,5")
        sel.mouseMoveAt("//tr[2]/td[1]", "5,5")
        sel.mouseUpAt("//tr[1]/td[1]", "5,5")
        # The dragged field is stored in the table below the field it was
        # dragged to.
        sel.clickAndWait("entity-fields-save")

        # After saving the position after drag'n'drop was stored.
        self.assertEqual(fieldtitle, sel.getText("//tr[2]/td[1]"))



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
# $Id: test_doctests.py 1369 2011-11-03 19:20:18Z icemac $
import icemac.addressbook.testing


def test_suite():
    return icemac.addressbook.testing.FunctionalDocFileSuite(
        'export.txt',
        'sortorder.txt',
        'userfields.txt',
        package='icemac.addressbook.export',
        )



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt

import os.path
import setuptools

def read(filename):
    return file(filename).read()

version = '1.8.0'

setuptools.setup(
    name='icemac.addressbook',
    version=version,
    description="Multi user address book application",
    long_description="\n\n".join([
        read('README.txt'),
        read('INSTALL.txt'),
        read('TODO.txt'),
        read('CHANGES.txt')] ),
    keywords='python address book addressbook zope3 zope application web '
             'phone number e-mail email home page homepage wsgi',
    author='Michael Howitz',
    author_email='icemac@gmx.net',
    url='http://pypi.python.org/pypi/icemac.addressbook',
    license='ZPL 2.1',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Paste',
        'Framework :: Zope3',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Other Audience',
        'Intended Audience :: Religion',
        'Intended Audience :: Telecommunications Industry',
        'License :: OSI Approved',
        'License :: OSI Approved :: Zope Public License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Topic :: Communications',
        'Topic :: Communications :: Email',
        'Topic :: Communications :: Email :: Address Book',
        'Topic :: Communications :: Telephony',
        'Topic :: Internet',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
        'Topic :: Office/Business',
        'Topic :: Office/Business :: Groupware',
        'Topic :: Religion',
        ],
    packages=setuptools.find_packages('src'),
    package_dir = {'': 'src'},
    namespace_packages = ['icemac'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'StableDict',
        'ZConfig',
        'ZODB3',
        'classproperty',
        'gocept.country',
        'gocept.pagelet',
        'gocept.reference',
        'hurry.jquery',
        'hurry.jqueryui',
        'hurry.resource',
        'hurry.zoperesource',
        'icemac.ab.locales [compile] >= 0.7',
        'icemac.truncatetext',
        'setuptools',
        'xlwt',
        'z3c.authviewlet',
        'z3c.flashmessage',
        'z3c.form >= 2.2.0dev',
        'z3c.formui',
        'z3c.layer.pagelet',
        'z3c.locales [compile] >= 0.3.0',
        'z3c.menu.ready2go',
        'z3c.pagelet',
        'z3c.preference',
        'z3c.table',
        'z3c.wizard',
        'zc.catalog',
        'zc.sourcefactory',
        'zdaemon',
        'zodbupdate',
        'zope.app.appsetup',
        'zope.app.debug',
        'zope.app.locales >= 3.6.0',
        'zope.app.principalannotation',
        'zope.app.wsgi',
        'zope.browserpage',
        'zope.copypastemove',
        'zope.generations',
        'zope.globalrequest',
        'zope.i18n [zcml,compile]',
        'zope.interface',
        'zope.login',
        'zope.mimetype',
        'zope.pluggableauth',
        'zope.preference',
        'zope.principalregistry',
        'zope.publisher',
        'zope.securitypolicy',
        'zope.session',
        'zope.traversing',
        ],
    extras_require = dict(
        test=[
            'gocept.selenium',
            'mock',
            'plone.testing',
            'unittest2',
            'xlrd',
            'z3c.etestbrowser',
            'zc.buildout',
            'zope.testbrowser[wsgi]>=4.0',
            'zope.testing >= 3.8',
            ],
        chameleon=[
            'z3c.pagelet[chameleon]',
            'z3c.ptcompat',
            ],
        ),
    entry_points = """
      [console_scripts]
      debug = icemac.addressbook.startup:interactive_debug_prompt
      addressbook = icemac.addressbook.startup:zdaemon_controller
      debug_ajax = icemac.addressbook.startup:zdaemon_controller_debug_ajax
      debug_pdb = icemac.addressbook.startup:zdaemon_controller_debug_pdb
      [paste.app_factory]
      main = icemac.addressbook.startup:application_factory
      """
    )



# Copyright (c) 2008-2011 Michael Howitz
# See also LICENSE.txt
# $Id: interfaces.py 1328 2011-09-22 19:42:28Z icemac $

from icemac.addressbook.i18n import _
import z3c.form.interfaces
import z3c.formui.interfaces
import z3c.layer.pagelet
import z3c.preference.interfaces
import zope.interface


class IAddressBookLayer(
    z3c.form.interfaces.IFormLayer,
    z3c.layer.pagelet.IPageletBrowserLayer,
    z3c.preference.interfaces.IPreferenceLayer):
    """Address book browser layer with form support."""


class IAddressBookBrowserSkin(
    z3c.formui.interfaces.IDivFormLayer,
    IAddressBookLayer):
    """The address book browser skin using the div-based layout."""


class IPersonCount(zope.interface.Interface):
    "Number of persons for deletion."

    count = zope.schema.Int(title=_(u'number of persons'), required=False)
    notes = zope.schema.TextLine(title=_(u'notes'), required=False)


class IErrorMessage(zope.interface.Interface):
    """Render error message human readable."""

    def __unicode__():
        """Returns the translateable error text."""



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2011 Michael Howitz
# See also LICENSE.txt

import icemac.addressbook.orderstorage
import unittest


class BaseTestOrderStorage(unittest.TestCase):

    def setUp(self):
        self.storage = icemac.addressbook.orderstorage.OrderStorage()


class TestOrderStorage(BaseTestOrderStorage):

    def test_add(self):
        self.assertEqual([], list(self.storage.namespaces()))

        self.storage.add('foo', 'bar')
        self.assertEqual(['bar'], list(self.storage.namespaces()))
        self.assertEqual(['foo'], list(self.storage.__iter__('bar')))

        self.storage.add('foo2', 'bar2')
        self.assertEqual(['bar', 'bar2'], sorted(self.storage.namespaces()))
        self.assertEqual(['foo'], list(self.storage.__iter__('bar')))
        self.assertEqual(['foo2'], list(self.storage.__iter__('bar2')))

        self.storage.add('foo2', 'bar')
        self.assertEqual(['bar', 'bar2'], sorted(self.storage.namespaces()))
        self.assertEqual(
            ['foo', 'foo2'], list(self.storage.__iter__('bar')))
        self.assertEqual(['foo2'], list(self.storage.__iter__('bar2')))

    def test_add_duplicate_no_error(self):
        self.storage.add('foo', 'bar')
        self.storage.add('foo', 'bar')
        self.assertEqual(['bar'], list(self.storage.namespaces()))
        self.assertEqual(['foo'], list(self.storage.__iter__('bar')))

    def test_add_duplicate_does_not_change_sortorder(self):
        self.storage.add('foo1', 'bar')
        self.storage.add('foo3', 'bar')
        self.storage.add('foo2', 'bar')
        self.storage.add('foo3', 'bar')
        self.assertEqual(
            ['foo1', 'foo3', 'foo2'], list(self.storage.__iter__('bar')))

    def test_add_duplicate_other_namespace(self):
        self.storage.add('foo', 'bar')
        self.storage.add('foo', 'bar2')
        self.assertEqual(
            ['bar', 'bar2'], sorted(self.storage.namespaces()))
        self.assertEqual(['foo'], list(self.storage.__iter__('bar')))
        self.assertEqual(['foo'], list(self.storage.__iter__('bar2')))

    def test_remove(self):
        self.storage.add('foo', 'bar')
        self.storage.add('foo2', 'bar2')
        self.storage.add('foo2', 'bar')

        self.storage.remove('foo', 'bar')
        self.assertEqual(['bar', 'bar2'], sorted(self.storage.namespaces()))
        self.assertEqual(['foo2'], list(self.storage.__iter__('bar')))
        self.assertEqual(['foo2'], list(self.storage.__iter__('bar2')))

        self.storage.remove('foo2', 'bar2')
        self.assertEqual(['bar', 'bar2'], sorted(self.storage.namespaces()))
        self.assertEqual(['foo2'], list(self.storage.__iter__('bar')))
        self.assertEqual([], list(self.storage.__iter__('bar2')))

        self.storage.remove('foo2', 'bar')
        self.assertEqual(['bar', 'bar2'], sorted(self.storage.namespaces()))
        self.assertEqual([], list(self.storage.__iter__('bar')))
        self.assertEqual([], list(self.storage.__iter__('bar2')))

    def test_remove_not_existing_namespace(self):
        self.storage.add('fuz', 'baz')
        self.assertRaises(KeyError, self.storage.remove, 'foo', 'bar')

    def test_remove_not_existing_obj(self):
        self.storage.add('fuz', 'bar')
        self.assertRaises(ValueError, self.storage.remove, 'foo', 'bar')

    def test_get__not_existing_namespace(self):
        self.assertRaises(KeyError, self.storage.get, 'foo1', 'bar')

    def test_get__not_existing_obj(self):
        self.storage.add('foo1', 'bar')
        self.assertRaises(KeyError, self.storage.get, 'foo2', 'bar')

    def test_get__existing_in_other_namespace(self):
        self.storage.add('foo1', 'baz')
        self.assertRaises(KeyError, self.storage.get, 'foo1', 'bar')

    def test_truncate_not_existing_namespace(self):
        self.storage.truncate('baz')
        # Truncation does not create a new namespace when it does not exist:
        self.assertEqual([], sorted(self.storage.namespaces()))

    def test_truncate_existing_namespace(self):
        self.storage.add('foo1', 'bar')
        self.storage.add('foo2', 'bar')
        self.storage.add('foo1', 'baz')
        self.storage.truncate('bar')
        # The specified namespace is empty, others are untouched:
        self.assertEqual([], list(self.storage.__iter__('bar')))
        self.assertEqual(['foo1'], list(self.storage.__iter__('baz')))


class TestOrderStorageMovement(BaseTestOrderStorage):

    def setUp(self):
        super(TestOrderStorageMovement, self).setUp()
        self.storage.add('foo1', 'bar')
        self.storage.add('foo2', 'bar')
        self.storage.add('foo3', 'bar')
        self.storage.add('foo4', 'bar')
        self.storage.add('baz', 'fuz')

    def test_up(self):
        self.assertEqual(['foo1', 'foo2', 'foo3', 'foo4'],
                         list(self.storage.__iter__('bar')))
        self.storage.up('foo3', 'bar')
        self.assertEqual(['foo1', 'foo3', 'foo2', 'foo4'],
                         list(self.storage.__iter__('bar')))
        self.assertEqual(['baz'], list(self.storage.__iter__('fuz')))
        self.assertEqual(1, self.storage.get('foo3', 'bar'))
        self.storage.up('foo3', 'bar')
        self.assertEqual(['foo3', 'foo1', 'foo2', 'foo4'],
                         list(self.storage.__iter__('bar')))
        self.assertEqual(0, self.storage.get('foo3', 'bar'))

    def test_up_first_element(self):
        self.assertRaises(ValueError, self.storage.up, 'foo1', 'bar')

    def test_down(self):
        self.storage.down('foo3', 'bar')
        self.assertEqual(['foo1', 'foo2', 'foo4', 'foo3'],
                         list(self.storage.__iter__('bar')))
        self.assertEqual(['baz'], list(self.storage.__iter__('fuz')))
        self.assertEqual(0, self.storage.get('foo1', 'bar'))
        self.storage.down('foo1', 'bar')
        self.assertEqual(['foo2', 'foo1', 'foo4', 'foo3'],
                         list(self.storage.__iter__('bar')))
        self.assertEqual(1, self.storage.get('foo1', 'bar'))

    def test_down_last_element(self):
        self.assertRaises(ValueError, self.storage.down, 'foo4', 'bar')

    def test_up_delta(self):
        self.storage.up('foo3', 'bar', 2)
        self.assertEqual(['foo3', 'foo1', 'foo2', 'foo4'],
                         list(self.storage.__iter__('bar')))

    def test_up_first_element_delta(self):
        self.assertRaises(ValueError, self.storage.up, 'foo2', 'bar', 2)

    def test_down_delta(self):
        self.storage.down('foo2', 'bar', 2)
        self.assertEqual(['foo1', 'foo3', 'foo4', 'foo2'],
                         list(self.storage.__iter__('bar')))

    def test_down_last_element_delta(self):
        self.assertRaises(ValueError, self.storage.down, 'foo3', 'bar', 2)

    def test_down_delta_0(self):
        self.storage.down('foo2', 'bar', 0)
        self.assertEqual(['foo1', 'foo2', 'foo3', 'foo4'],
                         list(self.storage.__iter__('bar')))

    def test_down_delta_negative(self):
        self.storage.down('foo2', 'bar', -2)
        self.assertEqual(['foo1', 'foo2', 'foo3', 'foo4'],
                         list(self.storage.__iter__('bar')))

    def test_get(self):
        self.storage.add('foo2', 'baz')
        self.assertEqual(1, self.storage.get('foo2', 'bar'))
        self.assertEqual(0, self.storage.get('foo2', 'baz'))

    def test_isFirst_not_in_List(self):
        self.assertRaises(KeyError, self.storage.isFirst, 'foo', 'fuz')

    def test_isFirst_not_first(self):
        self.assert_(self.storage.isFirst('foo2', 'bar') is False)
        self.assert_(self.storage.isFirst('foo3', 'bar') is False)
        self.assert_(self.storage.isFirst('foo4', 'bar') is False)

    def test_isFirst_first(self):
        self.assertTrue(self.storage.isFirst('foo1', 'bar'))
        self.assertTrue(self.storage.isFirst('baz', 'fuz'))

    def test_isLast_not_in_List(self):
        self.assertRaises(KeyError, self.storage.isLast, 'foo', 'fuz')

    def test_isLast_not_last(self):
        self.assert_(self.storage.isLast('foo1', 'bar') is False)
        self.assert_(self.storage.isLast('foo2', 'bar') is False)
        self.assert_(self.storage.isLast('foo3', 'bar') is False)

    def test_isLast_last(self):
        self.assertTrue(self.storage.isLast('foo4', 'bar'))
        self.assertTrue(self.storage.isLast('baz', 'fuz'))



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2011 Michael Howitz
# See also LICENSE.txt
import icemac.addressbook.entities
import icemac.addressbook.interfaces
import icemac.addressbook.testing
import unittest
import zope.component.testing
import zope.container.contained
import zope.container.sample
import zope.interface
import zope.schema
import zope.schema.interfaces


class IDummy(zope.interface.Interface):
    """Interface for test entity."""

    dummy = zope.schema.Text(title=u'dummy')
    dummy2 = zope.schema.Text(title=u'dummy2')


class Dummy(object):
    """Test entity."""
    zope.interface.implements(IDummy)


class TestEntity(unittest.TestCase):
    """Testing icemac.addressbook.entities.Entity."""

    def setUp(self):
        # Entities
        entities = zope.container.sample.SampleContainer()
        zope.interface.directlyProvides(
            entities, icemac.addressbook.interfaces.IEntities)
        zope.component.provideUtility(
            entities, icemac.addressbook.interfaces.IEntities)
        zope.component.provideAdapter(
            zope.container.contained.NameChooser,
            [icemac.addressbook.interfaces.IEntities])
        zope.component.provideAdapter(
            icemac.addressbook.entities.user_field_to_schema_field)

        # Order storage
        storage = icemac.addressbook.orderstorage.OrderStorage()
        zope.component.provideUtility(
            storage, icemac.addressbook.interfaces.IOrderStorage)

        # Entity under test
        self.entity = icemac.addressbook.entities.Entity(
            u'Dummy', IDummy, 'icemac.addressbook.tests.test_entity.Dummy')

        # Field under test
        self.user_field = icemac.addressbook.entities.Field()
        self.user_field.type = 'TextLine'
        self.entity.addField(self.user_field)
        self.schemaized_field = zope.schema.interfaces.IField(self.user_field)

    def tearDown(self):
        zope.component.testing.tearDown()

    # IEntityWrite

    def test_add_field_storage(self):
        entities = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntities)
        self.assertTrue(self.user_field is entities[u'Field'])

    def test_add_field_adapter_registration(self):
        field = zope.component.getMultiAdapter(
            (self.entity, Dummy()),
            icemac.addressbook.interfaces.IField, name=u'Field')
        self.assertTrue(self.user_field is field)

    def test_setFieldOrder_changing_initial_order(self):
        self.entity.setFieldOrder(['dummy2', 'Field', 'dummy'])
        self.assertEqual(['dummy2', 'Field', 'dummy'],
                         self.entity.getFieldOrder())

    def test_setFieldOrder_unkown_field_names(self):
        self.entity.setFieldOrder(
            ['dummy2', 'I-do-not-exist', 'dummy', 'Field'])
        self.assertEqual(['dummy2', 'dummy', 'Field'],
                         self.entity.getFieldOrder())
        # Unknown field names are not written into storage:
        order_storage = zope.component.getUtility(
            icemac.addressbook.interfaces.IOrderStorage)
        self.assertEqual(
            ['dummy2', 'dummy', 'Field'],
            order_storage.__iter__(self.entity._order_storage_namespace))

    def test_setFieldOrder_missing_field_names(self):
        self.entity.setFieldOrder(['Field', 'dummy'])
        # getFieldOrder only contains the values set by setFieldOrder
        self.assertEqual(['Field', 'dummy'],
                         self.entity.getFieldOrder())
        # When a field name is not in the field order it gets sorted to the
        # end:
        self.assertEqual(['Field', 'dummy', 'dummy2'],
                         [x[0] for x in self.entity.getRawFields()])

    def test_getFieldOrder_initial(self):
        # The field order is initially empty:
        self.assertEqual([], self.entity.getFieldOrder())

    def test_getFieldOrder_no_utility(self):
        # When the order utility cannot be found, the field order is empty,
        # too:
        zope.component.testing.tearDown()
        self.assertEqual([], self.entity.getFieldOrder())

    def test_getFieldOrder_namespace_not_computeable(self):
        # The namespace in the order utility depends on the name of the
        # entity which itself depends on the class_name stored on the
        # entity. But this class name is optional, so the name might not be
        # computable:
        entity = icemac.addressbook.entities.Entity(u'Dummy', IDummy, None)
        self.assertEqual([], entity.getFieldOrder())

    # IEntityRead

    def test_name(self):
        self.assertEqual('IcemacAddressbookTestsTestEntityDummy',
                         self.entity.name)

    def test_name_no_class_name_set(self):
        self.assertRaises(
            ValueError, getattr,
            icemac.addressbook.entities.Entity(None, IDummy, None), "name")

    def test_getRawFields(self):
        self.entity.setFieldOrder(['dummy2', 'Field', 'dummy'])
        self.assertEqual([('dummy2', IDummy['dummy2']),
                          ('Field', self.user_field),
                          ('dummy', IDummy['dummy'])],
                         list(self.entity.getRawFields()))

    def test_getRawFields_not_sorted(self):
        # When `sorted` is false the zope.schema fields are returned first
        # (the order is defined by the order in the interface) and then the
        # user defined fields (order is undefined here.)
        self.entity.setFieldOrder(['dummy2', 'Field', 'dummy'])
        self.assertEqual([('dummy', IDummy['dummy']),
                          ('dummy2', IDummy['dummy2']),
                          ('Field', self.user_field)],
                         list(self.entity.getRawFields(sorted=False)))

    def test_getFields(self):
        self.entity.setFieldOrder(['dummy2', 'Field', 'dummy'])
        self.assertEqual([('dummy2', IDummy['dummy2']),
                          ('Field', self.schemaized_field),
                          ('dummy', IDummy['dummy'])],
                         list(self.entity.getFields()))

    def test_getFields_not_sorted(self):
        # When `sorted` is false the zope.schema fields are returned first
        # (the order is defined by the order in the interface) and then the
        # user defined fields (order is undefined here.)
        self.entity.setFieldOrder(['dummy2', 'Field', 'dummy'])
        self.assertEqual([('dummy', IDummy['dummy']),
                          ('dummy2', IDummy['dummy2']),
                          ('Field', self.schemaized_field)],
                         list(self.entity.getFields(sorted=False)))

    def test_getFieldValues(self):
        self.entity.setFieldOrder(['dummy2', 'Field', 'dummy'])
        self.assertEqual(
            [IDummy['dummy2'], self.schemaized_field, IDummy['dummy']],
            self.entity.getFieldValues())

    def test_getFieldValues_not_sorted(self):
        self.entity.setFieldOrder(['dummy2', 'Field', 'dummy'])
        self.assertEqual(
            [IDummy['dummy'], IDummy['dummy2'], self.schemaized_field],
            self.entity.getFieldValues(sorted=False))

    def test_getField_unknown_field(self):
        self.assertRaises(KeyError, self.entity.getField, 'asdf')

    def test_getField_schema_field(self):
        self.assertEqual(IDummy['dummy2'], self.entity.getField('dummy2'))

    def test_getField_user_field(self):
        self.assertEqual(
            self.schemaized_field, self.entity.getField('Field'))

    def test_getRawField_unknown_field(self):
        self.assertRaises(KeyError, self.entity.getRawField, 'asdf')

    def test_getRawField_schema_field(self):
        self.assertEqual(IDummy['dummy2'], self.entity.getRawField('dummy2'))

    def test_getRawField_user_field(self):
        self.assertEqual(
            self.user_field, self.entity.getRawField('Field'))

    def test_getClass(self):
        self.assertEqual(Dummy, self.entity.getClass())

    def test_getClass_no_class_set(self):
        e = icemac.addressbook.entities.Entity(None, IDummy, None)
        self.assertRaises(ValueError, e.getClass)

    def test_tagged_values(self):
        e = icemac.addressbook.entities.Entity(
            u'Dummy', IDummy, 'Dummy', a=1, b='asdf')
        self.assertEqual(dict(a=1, b='asdf'), e.tagged_values)

    def test_tagged_values_returns_copy(self):
        # Tagged values not modifyable by modifying the returned dict.
        e = icemac.addressbook.entities.Entity(
            u'Dummy', IDummy, 'Dummy', a=1, b='asdf')
        e.tagged_values['a'] = 2
        self.assertEqual(dict(a=1, b='asdf'), e.tagged_values)


class Test_get_bound_schema_field(unittest.TestCase):
    """Testing icemac.addressbook.entities.get_bound_schema_field()."""

    layer = icemac.addressbook.testing.ADDRESS_BOOK_FUNCTIONAL_LAYER

    def callFUT(self, object, entity, field):
        from icemac.addressbook.entities import get_bound_schema_field
        return get_bound_schema_field(object, entity, field)

    def test_returns_field_if_object_provides_entity_interface(self):
        from icemac.addressbook.addressbook import address_book_entity
        import zope.schema

        ab = self.layer['addressbook']
        field = self.callFUT(ab, address_book_entity,
                             address_book_entity.getRawField('title'))
        self.assertEqual(ab, field.context)
        self.assertEqual('title', field.__name__)
        self.assertTrue(isinstance(field, zope.schema.TextLine))

    def test_looks_up_default_obj_on_obj_if_iface_not_provided_by_obj(self):
        from icemac.addressbook.address import postal_address_entity
        from icemac.addressbook.testing import create_full_person
        import zope.schema

        ab = self.layer['addressbook']
        person = create_full_person(ab, ab, u'Koch')
        field = self.callFUT(person, postal_address_entity,
                             postal_address_entity.getRawField('country'))
        self.assertEqual(person.default_postal_address, field.context)
        self.assertEqual('country', field.__name__)
        self.assertTrue(isinstance(field, zope.schema.Choice))

    def test_user_defined_fields_get_converted_to_schema_fields(self):
        from icemac.addressbook.address import phone_number_entity
        from icemac.addressbook.testing import create_full_person, create_field
        from icemac.addressbook.interfaces import IUserFieldStorage
        import zope.schema

        ab = self.layer['addressbook']
        person = create_full_person(ab, ab, u'Koch')
        create_field(ab, 'icemac.addressbook.address.PhoneNumber',
                     'Datetime', u'last call')
        field = self.callFUT(person, phone_number_entity,
                             phone_number_entity.getRawField('Field-1'))
        self.assertEqual(IUserFieldStorage(person.default_phone_number),
                         field.context)
        self.assertEqual('Field-1', field.__name__)
        self.assertTrue(isinstance(field, zope.schema.Datetime))
